from _typeshed import ConvertibleToInt

from qrcode import image as image
from qrcode.constants import (
    ERROR_CORRECT_H as ERROR_CORRECT_H,
    ERROR_CORRECT_L as ERROR_CORRECT_L,
    ERROR_CORRECT_M as ERROR_CORRECT_M,
    ERROR_CORRECT_Q as ERROR_CORRECT_Q,
)
from qrcode.main import GenericImage, QRCode as QRCode, make as make

from ._types import ErrorCorrect, MaskPattern

def run_example(
    data: str = "http://www.lincolnloop.com",
    version: ConvertibleToInt | None = None,
    error_correction: ErrorCorrect = 0,
    box_size: ConvertibleToInt = 10,
    border: ConvertibleToInt = 4,
    image_factory: type[GenericImage] | None = None,
    mask_pattern: MaskPattern | None = None,
) -> None: ...



from _typeshed import ReadableBuffer
from typing import Literal

__all__ = [
    "aes_cbc_decrypt",
    "aes_cbc_decrypt_bytes",
    "aes_cbc_encrypt",
    "aes_cbc_encrypt_bytes",
    "aes_ctr_decrypt",
    "aes_ctr_encrypt",
    "aes_decrypt",
    "aes_decrypt_text",
    "aes_ecb_decrypt",
    "aes_ecb_encrypt",
    "aes_encrypt",
    "aes_gcm_decrypt_and_verify",
    "aes_gcm_decrypt_and_verify_bytes",
    "key_expansion",
    "pad_block",
    "pkcs7_padding",
    "unpad_pkcs7",
]

def aes_cbc_decrypt_bytes(data: bytes, key: bytes, iv: bytes) -> bytes: ...
def aes_gcm_decrypt_and_verify_bytes(data: bytes, key: bytes, tag: bytes, nonce: bytes) -> bytes: ...
def aes_cbc_encrypt_bytes(
    data: bytes, key: bytes, iv: bytes, *, padding_mode: Literal["pkcs7", "iso7816", "whitespace", "zero"]
) -> bytes: ...
def unpad_pkcs7(data: list[int]) -> list[int]: ...
def pkcs7_padding(data: list[int]) -> list[int]: ...
def pad_block(block: list[int], padding_mode: Literal["pkcs7", "iso7816", "whitespace", "zero"]) -> list[int]: ...
def aes_ecb_encrypt(data: list[int], key: list[int], iv: list[int] | None = None) -> list[int]: ...
def aes_ecb_decrypt(data: list[int], key: list[int], iv: list[int] | None = None) -> list[int]: ...
def aes_ctr_decrypt(data: list[int], key: list[int], iv: list[int]) -> list[int]: ...
def aes_ctr_encrypt(data: list[int], key: list[int], iv: list[int]) -> list[int]: ...
def aes_cbc_decrypt(data: list[int], key: list[int], iv: list[int]) -> list[int]: ...
def aes_cbc_encrypt(data: list[int], key: list[int], iv: list[int], *, padding_mode: str = "pkcs7") -> list[int]: ...
def aes_gcm_decrypt_and_verify(data: list[int], key: list[int], tag: list[int], nonce: list[int]) -> list[int]: ...
def aes_encrypt(data: list[int], expanded_key: list[int]) -> list[int]: ...
def aes_decrypt(data: list[int], expanded_key: list[int]) -> list[int]: ...
def aes_decrypt_text(data: str | ReadableBuffer, password: str, key_size_bytes: int) -> str: ...
def key_expansion(data: list[int]) -> list[int]: ...



from braintree.graphql.enums.recommendations import Recommendations as Recommendations
from braintree.graphql.enums.recommended_payment_option import RecommendedPaymentOption as RecommendedPaymentOption



from typing import ClassVar, Final

from docutils import Component
from docutils.nodes import _Document

__docformat__: Final = "reStructuredText"

class Parser(Component):
    component_type: ClassVar[str]
    config_section: ClassVar[str]
    inputstring: str  # defined after call to setup_parse()
    document: _Document  # defined after call to setup_parse()
    def parse(self, inputstring: str, document: _Document) -> None: ...
    def setup_parse(self, inputstring: str, document: _Document) -> None: ...
    def finish_parse(self) -> None: ...

PARSER_ALIASES: Final[dict[str, str]]

def get_parser_class(parser_name: str) -> type[Parser]: ...



from _typeshed import Unused
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

import gdb

@type_check_only
class _Module(TypedDict):
    id: str | None
    name: str | None
    path: NotRequired[str | None]

@type_check_only
class _ModulesResult(TypedDict):
    modules: list[_Module]
    totalModules: int

def module_id(objfile: gdb.Objfile) -> str | None: ...
def is_module(objfile: gdb.Objfile) -> bool: ...
def make_module(objf: gdb.Objfile) -> _Module: ...
def modules(*, startModule: int = 0, moduleCount: int = 0, **args: Unused) -> _ModulesResult: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.attr_value_pb2
import tensorflow.core.protobuf.verifier_config_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AutoParallelOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLE_FIELD_NUMBER: builtins.int
    NUM_REPLICAS_FIELD_NUMBER: builtins.int
    enable: builtins.bool
    num_replicas: builtins.int
    def __init__(self, *, enable: builtins.bool | None = ..., num_replicas: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["enable", b"enable", "num_replicas", b"num_replicas"]) -> None: ...

global___AutoParallelOptions = AutoParallelOptions

@typing.final
class ScopedAllocatorOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLE_OP_FIELD_NUMBER: builtins.int
    @property
    def enable_op(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """If present, only perform optimization for these ops."""

    def __init__(self, *, enable_op: collections.abc.Iterable[builtins.str] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["enable_op", b"enable_op"]) -> None: ...

global___ScopedAllocatorOptions = ScopedAllocatorOptions

@typing.final
class RewriterConfig(google.protobuf.message.Message):
    """Graph rewriting is experimental and subject to change, not covered by any
    API stability guarantees.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Toggle:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ToggleEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RewriterConfig._Toggle.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: RewriterConfig._Toggle.ValueType  # 0
        ON: RewriterConfig._Toggle.ValueType  # 1
        OFF: RewriterConfig._Toggle.ValueType  # 2
        AGGRESSIVE: RewriterConfig._Toggle.ValueType  # 3
        """Enable some aggressive optimizations that use assumptions that TF graphs
        may break. For example, assume the shape of a placeholder matches its
        actual feed.
        """
        EXPERIMENTAL_MLIR: RewriterConfig._Toggle.ValueType  # 4
        """Run MLIR pass if there's one implemented in TFG, do nothing otherwise.
        I.e., if there's no corresponding TFG pass, it's an OFF. This is supposed
        to be mapped with `ON` and there's no `AGGRESSIVE` in MLIR pass now.
        """
        EXPERIMENTAL_BOTH: RewriterConfig._Toggle.ValueType  # 5
        """Run both MLIR and Grappler passes consecutively and MLIR pass will come
        first.
        """

    class Toggle(_Toggle, metaclass=_ToggleEnumTypeWrapper):
        """Configuration options for the meta-optimizer. Unless otherwise noted, these
        configuration options do not apply to explicitly triggered optimization
        passes in the optimizers field.
        """

    DEFAULT: RewriterConfig.Toggle.ValueType  # 0
    ON: RewriterConfig.Toggle.ValueType  # 1
    OFF: RewriterConfig.Toggle.ValueType  # 2
    AGGRESSIVE: RewriterConfig.Toggle.ValueType  # 3
    """Enable some aggressive optimizations that use assumptions that TF graphs
    may break. For example, assume the shape of a placeholder matches its
    actual feed.
    """
    EXPERIMENTAL_MLIR: RewriterConfig.Toggle.ValueType  # 4
    """Run MLIR pass if there's one implemented in TFG, do nothing otherwise.
    I.e., if there's no corresponding TFG pass, it's an OFF. This is supposed
    to be mapped with `ON` and there's no `AGGRESSIVE` in MLIR pass now.
    """
    EXPERIMENTAL_BOTH: RewriterConfig.Toggle.ValueType  # 5
    """Run both MLIR and Grappler passes consecutively and MLIR pass will come
    first.
    """

    class _CpuLayout:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CpuLayoutEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RewriterConfig._CpuLayout.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NO_CONVERSION_ON_CPU: RewriterConfig._CpuLayout.ValueType  # 0
        NCHW_TO_NHWC: RewriterConfig._CpuLayout.ValueType  # 1
        NHWC_TO_NCHW: RewriterConfig._CpuLayout.ValueType  # 2

    class CpuLayout(_CpuLayout, metaclass=_CpuLayoutEnumTypeWrapper):
        """Enum for layout conversion between NCHW and NHWC on CPU. Default is OFF."""

    NO_CONVERSION_ON_CPU: RewriterConfig.CpuLayout.ValueType  # 0
    NCHW_TO_NHWC: RewriterConfig.CpuLayout.ValueType  # 1
    NHWC_TO_NCHW: RewriterConfig.CpuLayout.ValueType  # 2

    class _NumIterationsType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _NumIterationsTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RewriterConfig._NumIterationsType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT_NUM_ITERS: RewriterConfig._NumIterationsType.ValueType  # 0
        ONE: RewriterConfig._NumIterationsType.ValueType  # 1
        TWO: RewriterConfig._NumIterationsType.ValueType  # 2

    class NumIterationsType(_NumIterationsType, metaclass=_NumIterationsTypeEnumTypeWrapper):
        """Enum controlling the number of times to run optimizers. The default is to
        run them twice.
        """

    DEFAULT_NUM_ITERS: RewriterConfig.NumIterationsType.ValueType  # 0
    ONE: RewriterConfig.NumIterationsType.ValueType  # 1
    TWO: RewriterConfig.NumIterationsType.ValueType  # 2

    class _MemOptType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _MemOptTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RewriterConfig._MemOptType.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT_MEM_OPT: RewriterConfig._MemOptType.ValueType  # 0
        """The default setting (SCHEDULING and SWAPPING HEURISTICS only)"""
        NO_MEM_OPT: RewriterConfig._MemOptType.ValueType  # 1
        """Disabled in the meta-optimizer."""
        MANUAL: RewriterConfig._MemOptType.ValueType  # 2
        """Driven by manual op-level annotations."""
        SWAPPING_HEURISTICS: RewriterConfig._MemOptType.ValueType  # 4
        """Driven by heuristics. The behavior of these heuristics is subject to
        change. Currently includes an experimental recomputation and swapping
        heuristics. Manual annotations are respected, but additional nodes are
        selected automatically.

        Swapping heuristic will move a tensor from the GPU to the CPU and move
        it back when needed to reduce peak memory usage.
        """
        RECOMPUTATION_HEURISTICS: RewriterConfig._MemOptType.ValueType  # 5
        """Recomputation heuristics will recompute ops (such as Relu activation)
        during backprop instead of storing them, reducing peak memory usage.
        """
        SCHEDULING_HEURISTICS: RewriterConfig._MemOptType.ValueType  # 6
        """Scheduling will split big ops such as AddN and try to enforce a schedule
        of the new computations that decreases peak memory usage.
        """
        HEURISTICS: RewriterConfig._MemOptType.ValueType  # 3
        """Use any combination of swapping and recomputation heuristics."""

    class MemOptType(_MemOptType, metaclass=_MemOptTypeEnumTypeWrapper): ...
    DEFAULT_MEM_OPT: RewriterConfig.MemOptType.ValueType  # 0
    """The default setting (SCHEDULING and SWAPPING HEURISTICS only)"""
    NO_MEM_OPT: RewriterConfig.MemOptType.ValueType  # 1
    """Disabled in the meta-optimizer."""
    MANUAL: RewriterConfig.MemOptType.ValueType  # 2
    """Driven by manual op-level annotations."""
    SWAPPING_HEURISTICS: RewriterConfig.MemOptType.ValueType  # 4
    """Driven by heuristics. The behavior of these heuristics is subject to
    change. Currently includes an experimental recomputation and swapping
    heuristics. Manual annotations are respected, but additional nodes are
    selected automatically.

    Swapping heuristic will move a tensor from the GPU to the CPU and move
    it back when needed to reduce peak memory usage.
    """
    RECOMPUTATION_HEURISTICS: RewriterConfig.MemOptType.ValueType  # 5
    """Recomputation heuristics will recompute ops (such as Relu activation)
    during backprop instead of storing them, reducing peak memory usage.
    """
    SCHEDULING_HEURISTICS: RewriterConfig.MemOptType.ValueType  # 6
    """Scheduling will split big ops such as AddN and try to enforce a schedule
    of the new computations that decreases peak memory usage.
    """
    HEURISTICS: RewriterConfig.MemOptType.ValueType  # 3
    """Use any combination of swapping and recomputation heuristics."""

    @typing.final
    class CustomGraphOptimizer(google.protobuf.message.Message):
        """Message to describe custom graph optimizer and its parameters"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class ParameterMapEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            @property
            def value(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue: ...
            def __init__(
                self, *, key: builtins.str | None = ..., value: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...
            ) -> None: ...
            def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

        NAME_FIELD_NUMBER: builtins.int
        PARAMETER_MAP_FIELD_NUMBER: builtins.int
        name: builtins.str
        @property
        def parameter_map(
            self,
        ) -> google.protobuf.internal.containers.MessageMap[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue]: ...
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            parameter_map: collections.abc.Mapping[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["name", b"name", "parameter_map", b"parameter_map"]) -> None: ...

    CPU_LAYOUT_CONVERSION_FIELD_NUMBER: builtins.int
    LAYOUT_OPTIMIZER_FIELD_NUMBER: builtins.int
    CONSTANT_FOLDING_FIELD_NUMBER: builtins.int
    SHAPE_OPTIMIZATION_FIELD_NUMBER: builtins.int
    REMAPPING_FIELD_NUMBER: builtins.int
    COMMON_SUBGRAPH_ELIMINATION_FIELD_NUMBER: builtins.int
    ARITHMETIC_OPTIMIZATION_FIELD_NUMBER: builtins.int
    DEPENDENCY_OPTIMIZATION_FIELD_NUMBER: builtins.int
    LOOP_OPTIMIZATION_FIELD_NUMBER: builtins.int
    FUNCTION_OPTIMIZATION_FIELD_NUMBER: builtins.int
    DEBUG_STRIPPER_FIELD_NUMBER: builtins.int
    DISABLE_MODEL_PRUNING_FIELD_NUMBER: builtins.int
    SCOPED_ALLOCATOR_OPTIMIZATION_FIELD_NUMBER: builtins.int
    PIN_TO_HOST_OPTIMIZATION_FIELD_NUMBER: builtins.int
    IMPLEMENTATION_SELECTOR_FIELD_NUMBER: builtins.int
    AUTO_MIXED_PRECISION_FIELD_NUMBER: builtins.int
    AUTO_MIXED_PRECISION_MKL_FIELD_NUMBER: builtins.int
    AUTO_MIXED_PRECISION_ONEDNN_BFLOAT16_FIELD_NUMBER: builtins.int
    AUTO_MIXED_PRECISION_CPU_FIELD_NUMBER: builtins.int
    DISABLE_META_OPTIMIZER_FIELD_NUMBER: builtins.int
    DISABLE_TFG_OPTIMIZER_FIELD_NUMBER: builtins.int
    USE_PLUGIN_OPTIMIZERS_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_CONDITIONAL_CODE_MOTION_FIELD_NUMBER: builtins.int
    META_OPTIMIZER_ITERATIONS_FIELD_NUMBER: builtins.int
    MIN_GRAPH_NODES_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_DISABLE_COMPRESSED_TENSOR_OPTIMIZATION_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_DISABLE_FOLDING_QUANTIZATION_EMULATION_FIELD_NUMBER: builtins.int
    MEMORY_OPTIMIZATION_FIELD_NUMBER: builtins.int
    MEMORY_OPTIMIZER_TARGET_NODE_NAME_SCOPE_FIELD_NUMBER: builtins.int
    META_OPTIMIZER_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    AUTO_PARALLEL_FIELD_NUMBER: builtins.int
    FAIL_ON_OPTIMIZER_ERRORS_FIELD_NUMBER: builtins.int
    SCOPED_ALLOCATOR_OPTS_FIELD_NUMBER: builtins.int
    OPTIMIZERS_FIELD_NUMBER: builtins.int
    CUSTOM_OPTIMIZERS_FIELD_NUMBER: builtins.int
    INTER_OPTIMIZER_VERIFIER_CONFIG_FIELD_NUMBER: builtins.int
    POST_OPTIMIZATION_VERIFIER_CONFIG_FIELD_NUMBER: builtins.int
    cpu_layout_conversion: global___RewriterConfig.CpuLayout.ValueType
    """CPU Conversion settings between NHCW and NCHW."""
    layout_optimizer: global___RewriterConfig.Toggle.ValueType
    """Optimize tensor layouts (default is ON)
    e.g. This will try to use NCHW layout on GPU which is faster.
    """
    constant_folding: global___RewriterConfig.Toggle.ValueType
    """Fold constants (default is ON)
    Statically infer the value of tensors when possible, and materialize the
    result using constants.
    """
    shape_optimization: global___RewriterConfig.Toggle.ValueType
    """Shape optimizations (default is ON)
    Simplify computations made on shapes.
    """
    remapping: global___RewriterConfig.Toggle.ValueType
    """Remapping (default is ON)
    Remap subgraphs onto more efficient implementations.
    """
    common_subgraph_elimination: global___RewriterConfig.Toggle.ValueType
    """Common subgraph elimination (default is ON)
    e.g. Simplify arithmetic ops; merge ops with same value (like constants).
    """
    arithmetic_optimization: global___RewriterConfig.Toggle.ValueType
    """Arithmetic optimizations (default is ON)
    e.g. Simplify arithmetic ops; merge ops with same value (like constants).
    """
    dependency_optimization: global___RewriterConfig.Toggle.ValueType
    """Control dependency optimizations (default is ON).
    Remove redundant control dependencies, which may enable other optimization.
    """
    loop_optimization: global___RewriterConfig.Toggle.ValueType
    """Loop optimizations (default is ON)."""
    function_optimization: global___RewriterConfig.Toggle.ValueType
    """Function optimizations (default is ON)."""
    debug_stripper: global___RewriterConfig.Toggle.ValueType
    """Strips debug-related nodes from the graph (off by default)."""
    disable_model_pruning: builtins.bool
    """If true, don't remove unnecessary ops from the graph"""
    scoped_allocator_optimization: global___RewriterConfig.Toggle.ValueType
    """Try to allocate some independent Op outputs contiguously in order to
    merge or eliminate downstream Ops (off by default).
    """
    pin_to_host_optimization: global___RewriterConfig.Toggle.ValueType
    """Force small ops onto the CPU (default is OFF)."""
    implementation_selector: global___RewriterConfig.Toggle.ValueType
    """Enable the swap of kernel implementations based on the device placement
    (default is ON).
    """
    auto_mixed_precision: global___RewriterConfig.Toggle.ValueType
    """Optimize data types for CUDA/oneDNN (default is OFF).
    This will try to use float16 on GPU/CPU which is faster.
    Note that this can change the numerical stability of the graph and may
    require the use of loss scaling to maintain model convergence.
    """
    auto_mixed_precision_mkl: global___RewriterConfig.Toggle.ValueType
    """Optimize data types for oneDNN (default is OFF).
    This will try to use bfloat16 on CPUs, which is faster.
    Note that this can change the numerical stability of the graph.
    Note: this is deprecated.
    It is replaced by auto_mixed_precision_onednn_bfloat16
    """
    auto_mixed_precision_onednn_bfloat16: global___RewriterConfig.Toggle.ValueType
    """Optimize data types for oneDNN (default is OFF).
    This will try to use bfloat16 on CPUs, which is faster.
    Note that this can change the numerical stability of the graph.
    Note: this is equivalent to the deprecated option auto_mixed_precision_mkl
    """
    auto_mixed_precision_cpu: global___RewriterConfig.Toggle.ValueType
    """Emulate a model using data type float16 on CPU (default is OFF).
    This will try to emulate the float16 inputs and outputs of an operator
    on CPU to have better correlation with float16 on GPU; however the
    computation in the operator is based on float32.
    Note that this can change the numerical stability of the graph.
    """
    disable_meta_optimizer: builtins.bool
    """Disable the entire meta optimizer (off by default)."""
    disable_tfg_optimizer: builtins.bool
    """Disable the TFG optimizer (off by default)."""
    use_plugin_optimizers: global___RewriterConfig.Toggle.ValueType
    """Optimizers registered by plugin (default is ON)"""
    experimental_conditional_code_motion: global___RewriterConfig.Toggle.ValueType
    """Conditional code motion (default is ON)."""
    meta_optimizer_iterations: global___RewriterConfig.NumIterationsType.ValueType
    """Controls how many times we run the optimizers in meta optimizer (default
    is once).
    """
    min_graph_nodes: builtins.int
    """The minimum number of nodes in a graph to optimizer. For smaller graphs,
    optimization is skipped.
    0 means the system picks an appropriate number.
    < 0 means do not skip optimization.
    """
    experimental_disable_compressed_tensor_optimization: builtins.bool
    """Disable optimizations that assume compressed tensors. Note that this flag
    is experimental and may be removed in the future.
    """
    experimental_disable_folding_quantization_emulation: builtins.bool
    """Disable folding quantization emulation ops such as FakeQuantWithMinMax* and
    QuantizeAndDequantize*. Some compilers (e.g. the TF-to-tflite converter)
    have to extract quantization configs (e.g. min/max range, number of bits,
    and per-channel) from the quantization emulation ops. Note that this flag
    is experimental and may be removed in the future. See b/174138564 for more
    details.
    """
    memory_optimization: global___RewriterConfig.MemOptType.ValueType
    """Configures memory optimization passes through the meta-optimizer. Has no
    effect on manually requested memory optimization passes in the optimizers
    field.
    """
    memory_optimizer_target_node_name_scope: builtins.str
    """A node name scope for node names which are valid outputs of recomputations.
    Inputs to nodes that match this scope may be recomputed (subject either to
    manual annotation of those input nodes or to manual annotation and
    heuristics depending on memory_optimization), but the nodes themselves will
    not be recomputed. This matches any sub-scopes as well, meaning the scope
    can appear not just as a top-level scope. For example, if the value is
    "gradients/", the default, it will match node name "gradients/foo",
    "foo/gradients/bar", but not "foo_gradients/"
    """
    meta_optimizer_timeout_ms: builtins.int
    """Maximum number of milliseconds to spend optimizing a single graph before
    timing out. If less than or equal to 0 (default value) the optimizer will
    never time out.
    """
    fail_on_optimizer_errors: builtins.bool
    """If true, any optimization pass failing will cause the MetaOptimizer to
    stop with an error. By default - or when set to false, failing passes are
    skipped silently.
    """
    @property
    def auto_parallel(self) -> global___AutoParallelOptions:
        """Configures AutoParallel optimization passes either through the
        meta-optimizer or when manually specified through the optimizers field.
        """

    @property
    def scoped_allocator_opts(self) -> global___ScopedAllocatorOptions: ...
    @property
    def optimizers(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """If non-empty, will use this as an alternative way to specify a list of
        optimizations to turn on and the order of the optimizations (replacing the
        meta-optimizer).

        Of the RewriterConfig options, only the AutoParallel configuration options
        (the auto_parallel field) apply to manually requested optimization passes
        ("autoparallel"). Memory optimization passes ("memory") invoked here are
        not configurable (in contrast to memory optimization passes through the
        meta-optimizer) and act only on manual op annotations.

        Custom optimizers (see custom_optimizers) that are not part of this
        schedule will be run after - in the order that they were specified.
        """

    @property
    def custom_optimizers(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RewriterConfig.CustomGraphOptimizer]:
        """list of CustomGraphOptimizers to apply."""

    @property
    def inter_optimizer_verifier_config(self) -> tensorflow.core.protobuf.verifier_config_pb2.VerifierConfig:
        """VerifierConfig specifying the verifiers to be run after every optimizer."""

    @property
    def post_optimization_verifier_config(self) -> tensorflow.core.protobuf.verifier_config_pb2.VerifierConfig:
        """VerifierConfig specifying the verifiers to be run at the end, after all
        optimizers have run.
        """

    def __init__(
        self,
        *,
        cpu_layout_conversion: global___RewriterConfig.CpuLayout.ValueType | None = ...,
        layout_optimizer: global___RewriterConfig.Toggle.ValueType | None = ...,
        constant_folding: global___RewriterConfig.Toggle.ValueType | None = ...,
        shape_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        remapping: global___RewriterConfig.Toggle.ValueType | None = ...,
        common_subgraph_elimination: global___RewriterConfig.Toggle.ValueType | None = ...,
        arithmetic_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        dependency_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        loop_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        function_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        debug_stripper: global___RewriterConfig.Toggle.ValueType | None = ...,
        disable_model_pruning: builtins.bool | None = ...,
        scoped_allocator_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        pin_to_host_optimization: global___RewriterConfig.Toggle.ValueType | None = ...,
        implementation_selector: global___RewriterConfig.Toggle.ValueType | None = ...,
        auto_mixed_precision: global___RewriterConfig.Toggle.ValueType | None = ...,
        auto_mixed_precision_mkl: global___RewriterConfig.Toggle.ValueType | None = ...,
        auto_mixed_precision_onednn_bfloat16: global___RewriterConfig.Toggle.ValueType | None = ...,
        auto_mixed_precision_cpu: global___RewriterConfig.Toggle.ValueType | None = ...,
        disable_meta_optimizer: builtins.bool | None = ...,
        disable_tfg_optimizer: builtins.bool | None = ...,
        use_plugin_optimizers: global___RewriterConfig.Toggle.ValueType | None = ...,
        experimental_conditional_code_motion: global___RewriterConfig.Toggle.ValueType | None = ...,
        meta_optimizer_iterations: global___RewriterConfig.NumIterationsType.ValueType | None = ...,
        min_graph_nodes: builtins.int | None = ...,
        experimental_disable_compressed_tensor_optimization: builtins.bool | None = ...,
        experimental_disable_folding_quantization_emulation: builtins.bool | None = ...,
        memory_optimization: global___RewriterConfig.MemOptType.ValueType | None = ...,
        memory_optimizer_target_node_name_scope: builtins.str | None = ...,
        meta_optimizer_timeout_ms: builtins.int | None = ...,
        auto_parallel: global___AutoParallelOptions | None = ...,
        fail_on_optimizer_errors: builtins.bool | None = ...,
        scoped_allocator_opts: global___ScopedAllocatorOptions | None = ...,
        optimizers: collections.abc.Iterable[builtins.str] | None = ...,
        custom_optimizers: collections.abc.Iterable[global___RewriterConfig.CustomGraphOptimizer] | None = ...,
        inter_optimizer_verifier_config: tensorflow.core.protobuf.verifier_config_pb2.VerifierConfig | None = ...,
        post_optimization_verifier_config: tensorflow.core.protobuf.verifier_config_pb2.VerifierConfig | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "auto_parallel",
            b"auto_parallel",
            "inter_optimizer_verifier_config",
            b"inter_optimizer_verifier_config",
            "post_optimization_verifier_config",
            b"post_optimization_verifier_config",
            "scoped_allocator_opts",
            b"scoped_allocator_opts",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "arithmetic_optimization",
            b"arithmetic_optimization",
            "auto_mixed_precision",
            b"auto_mixed_precision",
            "auto_mixed_precision_cpu",
            b"auto_mixed_precision_cpu",
            "auto_mixed_precision_mkl",
            b"auto_mixed_precision_mkl",
            "auto_mixed_precision_onednn_bfloat16",
            b"auto_mixed_precision_onednn_bfloat16",
            "auto_parallel",
            b"auto_parallel",
            "common_subgraph_elimination",
            b"common_subgraph_elimination",
            "constant_folding",
            b"constant_folding",
            "cpu_layout_conversion",
            b"cpu_layout_conversion",
            "custom_optimizers",
            b"custom_optimizers",
            "debug_stripper",
            b"debug_stripper",
            "dependency_optimization",
            b"dependency_optimization",
            "disable_meta_optimizer",
            b"disable_meta_optimizer",
            "disable_model_pruning",
            b"disable_model_pruning",
            "disable_tfg_optimizer",
            b"disable_tfg_optimizer",
            "experimental_conditional_code_motion",
            b"experimental_conditional_code_motion",
            "experimental_disable_compressed_tensor_optimization",
            b"experimental_disable_compressed_tensor_optimization",
            "experimental_disable_folding_quantization_emulation",
            b"experimental_disable_folding_quantization_emulation",
            "fail_on_optimizer_errors",
            b"fail_on_optimizer_errors",
            "function_optimization",
            b"function_optimization",
            "implementation_selector",
            b"implementation_selector",
            "inter_optimizer_verifier_config",
            b"inter_optimizer_verifier_config",
            "layout_optimizer",
            b"layout_optimizer",
            "loop_optimization",
            b"loop_optimization",
            "memory_optimization",
            b"memory_optimization",
            "memory_optimizer_target_node_name_scope",
            b"memory_optimizer_target_node_name_scope",
            "meta_optimizer_iterations",
            b"meta_optimizer_iterations",
            "meta_optimizer_timeout_ms",
            b"meta_optimizer_timeout_ms",
            "min_graph_nodes",
            b"min_graph_nodes",
            "optimizers",
            b"optimizers",
            "pin_to_host_optimization",
            b"pin_to_host_optimization",
            "post_optimization_verifier_config",
            b"post_optimization_verifier_config",
            "remapping",
            b"remapping",
            "scoped_allocator_optimization",
            b"scoped_allocator_optimization",
            "scoped_allocator_opts",
            b"scoped_allocator_opts",
            "shape_optimization",
            b"shape_optimization",
            "use_plugin_optimizers",
            b"use_plugin_optimizers",
        ],
    ) -> None: ...

global___RewriterConfig = RewriterConfig



import abc
from _typeshed import Unused
from collections import defaultdict
from collections.abc import Generator
from contextlib import AbstractContextManager
from typing import TypedDict, type_check_only
from typing_extensions import NotRequired

import gdb

@type_check_only
class _ValueFormat(TypedDict, total=False):
    hex: bool

@type_check_only
class _ReferenceDescriptor(TypedDict):
    # Result of BaseReference.to_object()
    variableReference: int
    name: NotRequired[str]

@type_check_only
class _VariableReferenceDescriptor(_ReferenceDescriptor):
    # Result of VariableReference.to_object()
    indexedVariables: NotRequired[int]
    namedVariables: NotRequired[int]
    memoryReference: NotRequired[str]
    type: NotRequired[str]
    # Below key name set by VariableReference.result_name
    # Could be modelled with extra_items=str if PEP 728 is accepted.
    value: NotRequired[str]

all_variables: list[BaseReference]

def clear_vars(event: Unused) -> None: ...
def apply_format(value_format: _ValueFormat | None) -> AbstractContextManager[None]: ...

class BaseReference(abc.ABC):
    ref: int
    name: str
    children: list[VariableReference | None] | None
    by_name: dict[str, VariableReference]
    name_counts: defaultdict[str, int]
    def __init__(self, name: str) -> None: ...
    def to_object(self) -> _ReferenceDescriptor: ...
    @abc.abstractmethod
    def has_children(self) -> bool: ...
    def reset_children(self): ...
    @abc.abstractmethod
    def fetch_one_child(self, index: int) -> tuple[str, gdb.Value]: ...
    @abc.abstractmethod
    def child_count(self) -> int: ...
    def fetch_children(self, start: int, count: int) -> Generator[VariableReference]: ...
    def find_child_by_name(self, name: str) -> VariableReference: ...

class VariableReference(BaseReference):
    result_name: str
    value: gdb.Value
    child_cache: list[tuple[int | str, gdb.Value]] | None
    count: int | None
    printer: gdb._PrettyPrinter
    def __init__(self, name: str, value: gdb.Value, result_name: str = "value") -> None: ...
    def assign(self, value: gdb.Value) -> None: ...
    def has_children(self) -> bool: ...
    def cache_children(self) -> list[tuple[int | str, gdb.Value]]: ...
    def child_count(self) -> int: ...
    def to_object(self) -> _VariableReferenceDescriptor: ...
    # note: parameter named changed from 'index' to 'idx'
    def fetch_one_child(self, idx: int) -> tuple[str, gdb.Value]: ...

def find_variable(ref: int) -> BaseReference: ...



import tensorflow as tf
from tensorflow._aliases import Integer, StringTensorCompatible

def decode_wav(
    contents: StringTensorCompatible, desired_channels: int = -1, desired_samples: int = -1, name: str | None = None
) -> tuple[tf.Tensor, tf.Tensor]: ...
def encode_wav(audio: tf.Tensor, sample_rate: Integer, name: str | None = None) -> tf.Tensor: ...



from collections.abc import Callable, MutableMapping, Sequence
from typing import Any, Final, Literal, overload
from typing_extensions import deprecated

import click

__all__ = ["DefaultGroup"]
__version__: Final[str]

class DefaultGroup(click.Group):
    ignore_unknown_options: bool
    default_cmd_name: str | None
    default_if_no_args: bool
    # type hints were taken from click lib
    def __init__(
        self,
        name: str | None = None,
        commands: MutableMapping[str, click.Command] | Sequence[click.Command] | None = None,
        *,
        ignore_unknown_options: Literal[True] | None = True,
        default: str | None = None,
        default_if_no_args: bool = False,
        invoke_without_command: bool = False,
        no_args_is_help: bool | None = None,
        subcommand_metavar: str | None = None,
        chain: bool = False,
        result_callback: Callable[..., Any] | None = None,  # Any is specified in click lib
        context_settings: MutableMapping[str, Any] | None = None,  # Any is specified in click lib
        callback: Callable[..., Any] | None = None,  # Any is specified in click lib
        params: list[click.Parameter] | None = None,
        help: str | None = None,
        epilog: str | None = None,
        short_help: str | None = None,
        options_metavar: str | None = "[OPTIONS]",
        add_help_option: bool = True,
        hidden: bool = False,
        deprecated: bool = False,
    ) -> None: ...
    def set_default_command(self, command: click.Command) -> None: ...
    def parse_args(self, ctx: click.Context, args: list[str]) -> list[str]: ...
    def get_command(self, ctx: click.Context, cmd_name: str) -> click.Command | None: ...
    def resolve_command(self, ctx: click.Context, args: list[str]) -> tuple[str | None, click.Command | None, list[str]]: ...
    def format_commands(self, ctx: click.Context, formatter: click.HelpFormatter) -> None: ...
    @overload
    def command(
        self,
        __func: Callable[..., Any],
        /,
        *,
        name: str | None = ...,
        cls: type[click.Command] | None = ...,
        default: Literal[False] = False,
    ) -> click.Command: ...
    @overload
    @deprecated("Use default param of `DefaultGroup` or `set_default_command()` instead")
    def command(
        self,
        __func: Callable[..., Any],
        /,
        *,
        name: str | None = ...,
        cls: type[click.Command] | None = ...,
        default: Literal[True],
    ) -> click.Command: ...
    @overload
    def command(
        self, *, name: str | None = ..., cls: type[click.Command] | None = ..., default: Literal[False] = False
    ) -> Callable[[Callable[..., Any]], click.Command]: ...
    @overload
    @deprecated("Use default param of `DefaultGroup` or `set_default_command()` instead")
    def command(
        self, *, name: str | None = ..., cls: type[click.Command] | None = ..., default: Literal[True]
    ) -> Callable[[Callable[..., Any]], click.Command]: ...
    @overload
    def command(self, *args: Any, **kwargs: Any) -> Callable[[Callable[..., Any]], click.Command] | click.Command: ...

class DefaultCommandFormatter:
    group: click.Group
    formatter: click.HelpFormatter
    mark: str
    def __init__(self, group: click.Group, formatter: click.HelpFormatter, mark: str = "*") -> None: ...
    def write_dl(self, rows: Sequence[tuple[str, str]], col_max: int = 30, col_spacing: int = -2) -> None: ...
    def __getattr__(self, attr: str) -> Any: ...  # attribute access is forwarded to click.HelpFormatter



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["weisfeiler_lehman_graph_hash", "weisfeiler_lehman_subgraph_hashes"]

@_dispatchable
def weisfeiler_lehman_graph_hash(
    G: Graph[_Node],
    edge_attr: str | None = None,
    node_attr: str | None = None,
    iterations: int | None = 3,
    digest_size: int | None = 16,
) -> str: ...
@_dispatchable
def weisfeiler_lehman_subgraph_hashes(
    G: Graph[_Node],
    edge_attr: str | None = None,
    node_attr: str | None = None,
    iterations: int | None = 3,
    digest_size: int | None = 16,
    include_initial_labels: bool | None = False,
) -> dict[Incomplete, list[str]]: ...



from __future__ import annotations

import distutils.command.sdist
from _typeshed import StrPath
from os import PathLike
from pathlib import Path

from setuptools._distutils.ccompiler import CCompiler

c = distutils.command.sdist.sdist

# Test CCompiler().compile with varied sources

compiler = CCompiler()

str_list: list[str] = ["file1.c", "file2.c"]
compiler.compile(sources=str_list)

path_list: list[Path] = [Path("file1.c"), Path("file2.c")]
compiler.compile(sources=path_list)

pathlike_list: list[PathLike[str]] = [Path("file1.c"), Path("file2.c")]
compiler.compile(sources=pathlike_list)

strpath_list: list[StrPath] = [Path("file1.c"), "file2.c"]
compiler.compile(sources=strpath_list)

# Direct literals should also work
compiler.compile(sources=["file1.c", "file2.c"])
compiler.compile(sources=[Path("file1.c"), Path("file2.c")])
compiler.compile(sources=[Path("file1.c"), "file2.c"])



from collections.abc import Container, Iterable, Mapping
from typing import Any

from .backends.base import Key

def sign(
    payload: bytes | Mapping[str, Any],
    # Internally it's passed down to jwk.construct(), which explicitly checks for
    # key as dict instance, instead of a Mapping
    key: str | bytes | dict[str, Any] | Key,
    headers: Mapping[str, Any] | None = None,
    algorithm: str = "HS256",
) -> str: ...
def verify(
    token: str | bytes,
    key: str | bytes | Mapping[str, Any] | Key | Iterable[str],
    # Callers of this function, like jwt.decode(), and functions called internally,
    # like jws._verify_signature(), use and accept algorithms=None
    algorithms: str | Container[str] | None,
    verify: bool = True,
) -> bytes: ...
def get_unverified_header(token: str | bytes) -> dict[str, Any]: ...
def get_unverified_headers(token: str | bytes) -> dict[str, Any]: ...
def get_unverified_claims(token: str | bytes) -> bytes: ...



from abc import ABC, abstractmethod
from collections.abc import Callable
from typing import Any, Final, Literal, TypeVar, overload
from typing_extensions import Self, TypeAlias, TypeGuard

from tensorflow import Tensor
from tensorflow._aliases import KerasSerializable, TensorCompatible
from tensorflow.keras.metrics import (
    binary_crossentropy as binary_crossentropy,
    categorical_crossentropy as categorical_crossentropy,
)

class Loss(ABC):
    reduction: _ReductionValues
    name: str | None
    def __init__(self, name: str | None = None, reduction: _ReductionValues = "sum_over_batch_size", dtype=None) -> None: ...
    @abstractmethod
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...
    def get_config(self) -> dict[str, Any]: ...
    def __call__(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Tensor: ...

class BinaryCrossentropy(Loss):
    def __init__(
        self,
        from_logits: bool = False,
        label_smoothing: float = 0.0,
        axis: int = -1,
        reduction: _ReductionValues = "sum_over_batch_size",
        name: str | None = "binary_crossentropy",
        dtype=None,
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class BinaryFocalCrossentropy(Loss):
    def __init__(
        self,
        apply_class_balancing: bool = False,
        alpha: float = 0.25,
        gamma: float = 2.0,
        from_logits: bool = False,
        label_smoothing: float = 0.0,
        axis: int = -1,
        reduction: _ReductionValues = "sum_over_batch_size",
        name: str | None = "binary_focal_crossentropy",
        dtype=None,
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class CategoricalCrossentropy(Loss):
    def __init__(
        self,
        from_logits: bool = False,
        label_smoothing: float = 0.0,
        axis: int = -1,
        reduction: _ReductionValues = "sum_over_batch_size",
        name: str | None = "categorical_crossentropy",
        dtype=None,
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class CategoricalHinge(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "categorical_hinge", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class CosineSimilarity(Loss):
    def __init__(
        self,
        axis: int = -1,
        reduction: _ReductionValues = "sum_over_batch_size",
        name: str | None = "cosine_similarity",
        dtype=None,
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class Hinge(Loss):
    def __init__(self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "hinge", dtype=None) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class Huber(Loss):
    def __init__(
        self, delta: float = 1.0, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "huber_loss", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class KLDivergence(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "kl_divergence", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class LogCosh(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "log_cosh", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class MeanAbsoluteError(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "mean_absolute_error", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class MeanAbsolutePercentageError(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "mean_absolute_percentage_error", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class MeanSquaredError(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "mean_squared_error", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class MeanSquaredLogarithmicError(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "mean_squared_logarithmic_error", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class Poisson(Loss):
    def __init__(self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "poisson", dtype=None) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class SparseCategoricalCrossentropy(Loss):
    def __init__(
        self,
        from_logits: bool = False,
        ignore_class: int | None = None,
        reduction: _ReductionValues = "sum_over_batch_size",
        name: str = "sparse_categorical_crossentropy",
        dtype=None,
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class SquaredHinge(Loss):
    def __init__(
        self, reduction: _ReductionValues = "sum_over_batch_size", name: str | None = "squared_hinge", dtype=None
    ) -> None: ...
    def call(self, y_true: Tensor, y_pred: Tensor) -> Tensor: ...

class Reduction:
    AUTO: Final = "auto"
    NONE: Final = "none"
    SUM: Final = "sum"
    SUM_OVER_BATCH_SIZE: Final = "sum_over_batch_size"
    @classmethod
    def all(cls) -> tuple[_ReductionValues, ...]: ...
    @classmethod
    def validate(cls, key: object) -> TypeGuard[_ReductionValues]: ...

_ReductionValues: TypeAlias = Literal["auto", "none", "sum", "sum_over_batch_size"]

def categorical_hinge(y_true: TensorCompatible, y_pred: TensorCompatible) -> Tensor: ...
def huber(y_true: TensorCompatible, y_pred: TensorCompatible, delta: float = 1.0) -> Tensor: ...
def deserialize(name: str | dict[str, Any], custom_objects: dict[str, Any] | None = None) -> Loss: ...
def serialize(loss: KerasSerializable) -> dict[str, Any]: ...

_FuncT = TypeVar("_FuncT", bound=Callable[..., Any])

@overload
def get(identifier: None) -> None: ...
@overload
def get(identifier: str | dict[str, Any]) -> Loss: ...
@overload
def get(identifier: _FuncT) -> _FuncT: ...

# This is complete with respect to methods documented defined here,
# but many methods get re-exported here from tf.keras.metrics that aren't
# covered yet.
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete, Unused
from collections import defaultdict
from collections.abc import Iterable, Iterator

from .encryption import StandardSecurityHandler
from .syntax import PDFArray, PDFObject, PDFString

class NumberTree(PDFObject):
    __slots__ = ("_id", "nums")
    nums: defaultdict[Incomplete, list[Incomplete]]
    def __init__(self) -> None: ...
    def serialize(self, obj_dict: Unused = None, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class StructTreeRoot(PDFObject):
    __slots__ = ("_id", "type", "parent_tree", "k")
    type: str
    parent_tree: NumberTree
    k: PDFArray[Incomplete]
    def __init__(self) -> None: ...

class StructElem(PDFObject):
    __slots__ = ("_id", "type", "s", "p", "k", "t", "alt", "pg", "_page_number")
    type: str
    s: str
    p: PDFObject
    k: PDFArray[Incomplete]
    t: PDFString | None
    alt: PDFString | None
    pg: Incomplete | None
    def __init__(
        self,
        struct_type: str,
        parent: PDFObject,
        kids: Iterable[int] | Iterable[StructElem],
        page_number: int | None = None,
        title: str | None = None,
        alt: str | None = None,
    ) -> None: ...
    def page_number(self) -> int | None: ...

class StructureTreeBuilder:
    struct_tree_root: Incomplete
    doc_struct_elem: Incomplete
    struct_elem_per_mc: Incomplete
    def __init__(self) -> None: ...
    def add_marked_content(
        self, page_number: int, struct_type: str, mcid: int | None = None, title: str | None = None, alt_text: str | None = None
    ) -> tuple[Incomplete, Incomplete]: ...
    def next_mcid_for_page(self, page_number: int) -> int: ...
    def empty(self) -> bool: ...
    def __iter__(self) -> Iterator[Incomplete]: ...



from typing import Final

from google.protobuf.descriptor_pb2 import FileDescriptorProto

__author__: Final[str]

class Error(Exception): ...
class DescriptorDatabaseConflictingDefinitionError(Error): ...

class DescriptorDatabase:
    def __init__(self) -> None: ...
    def Add(self, file_desc_proto) -> None: ...
    def FindFileByName(self, name): ...
    def FindFileContainingSymbol(self, symbol): ...
    def FindFileContainingExtension(self, extendee_name, extension_number) -> FileDescriptorProto | None: ...
    def FindAllExtensionNumbers(self, extendee_name) -> list[int]: ...



from abc import abstractmethod
from collections.abc import Sequence
from typing import Any
from typing_extensions import Self

import tensorflow as tf

class LearningRateSchedule:
    # At runtime these methods are abstract even though class is not ABC.
    @abstractmethod
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    @abstractmethod
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class PiecewiseConstantDecay(LearningRateSchedule):
    def __init__(
        self,
        boundaries: Sequence[tf.Tensor] | Sequence[float],
        values: Sequence[float] | Sequence[tf.Tensor],
        name: str = "PiecewiseConstant",
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class InverseTimeDecay(LearningRateSchedule):
    def __init__(
        self,
        initial_learning_rate: float | tf.Tensor,
        decay_steps: int,
        decay_rate: float,
        staircase: bool = False,
        name: str = "InverseTimeDecay",
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class PolynomialDecay(LearningRateSchedule):
    def __init__(
        self,
        initial_learning_rate: float | tf.Tensor,
        decay_steps: int,
        end_learning_rate: float | tf.Tensor = 0.0001,
        power: float = 1.0,
        cycle: bool = False,
        name: str = "PolynomialDecay",
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class CosineDecay(LearningRateSchedule):
    def __init__(
        self,
        initial_learning_rate: float | tf.Tensor,
        decay_steps: int,
        alpha: float | tf.Tensor = 0.0,
        name: str = "CosineDecay",
        warmup_target: int | tf.Tensor | None = None,  # float32 or float64 Tensor
        warmup_steps: int | tf.Tensor = 0,  # int32 or int64 Tensor
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class CosineDecayRestarts(LearningRateSchedule):
    def __init__(
        self,
        initial_learning_rate: float | tf.Tensor,
        first_decay_steps: int | tf.Tensor,
        t_mul: float | tf.Tensor = 2.0,
        m_mul: float | tf.Tensor = 1.0,
        alpha: float | tf.Tensor = 0.0,
        name: str = "SGDRDecay",
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

class ExponentialDecay(LearningRateSchedule):
    def __init__(
        self,
        initial_learning_rate: float | tf.Tensor,
        decay_steps: int | tf.Tensor,
        decay_rate: float | tf.Tensor,
        staircase: bool = False,
        name: str = "ExponentialDecay",
    ) -> None: ...
    def __call__(self, step: int | tf.Tensor) -> float | tf.Tensor: ...
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...

def deserialize(config: dict[str, Any], custom_objects: dict[str, type] | None = None) -> LearningRateSchedule: ...
def serialize(learning_rate_schedule: LearningRateSchedule) -> dict[str, Any]: ...



from collections.abc import Iterable
from typing import Any

from google.protobuf.descriptor import Descriptor
from google.protobuf.descriptor_pb2 import FileDescriptorProto
from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

class MessageFactory:
    pool: Any
    def __init__(self, pool: DescriptorPool | None = None) -> None: ...

def GetMessageClass(descriptor: Descriptor) -> type[Message]: ...
def GetMessageClassesForFiles(files: Iterable[str], pool: DescriptorPool) -> dict[str, type[Message]]: ...
def GetMessages(file_protos: Iterable[FileDescriptorProto], pool: DescriptorPool | None = None) -> dict[str, type[Message]]: ...



import subprocess
import sys

from ts_utils.requirements import get_external_stub_requirements


def main() -> None:
    requirements = get_external_stub_requirements()
    # By forwarding arguments, we naturally allow non-venv (system installs)
    # by letting the script's user follow uv's own helpful hint of passing the `--system` flag.
    subprocess.check_call(["uv", "pip", "install", *sys.argv[1:], *[str(requirement) for requirement in requirements]])


if __name__ == "__main__":
    main()



from _typeshed import Incomplete
from collections.abc import Callable, Collection, Hashable, Iterable
from typing import Literal, TypeVar, overload
from typing_extensions import TypeAlias

import numpy
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

# stub_uploader won't allow pandas-stubs in the requires field https://github.com/typeshed-internal/stub_uploader/issues/90
# from pandas import DataFrame
_DataFrame: TypeAlias = Incomplete
# from pandas.core.dtypes.base import ExtensionDtype
_ExtensionDtype: TypeAlias = Incomplete
# pandas._typing import Axes
# _Axes: TypeAlias = Index | Series | np.ndarray | list | dict | range | tuple
_Axes: TypeAlias = Collection[_Node]
_G = TypeVar("_G", bound=Graph[Hashable])

__all__ = [
    "from_pandas_adjacency",
    "to_pandas_adjacency",
    "from_pandas_edgelist",
    "to_pandas_edgelist",
    "from_scipy_sparse_array",
    "to_scipy_sparse_array",
    "from_numpy_array",
    "to_numpy_array",
]

@_dispatchable
def to_pandas_adjacency(
    G: Graph[_Node],
    nodelist: _Axes[_Node] | None = None,
    dtype: numpy.dtype[Incomplete] | None = None,
    order: numpy._OrderCF = None,
    multigraph_weight: Callable[[list[float]], float] = ...,
    weight: str = "weight",
    nonedge: float = 0.0,
) -> _DataFrame: ...
@overload
def from_pandas_adjacency(df: _DataFrame, create_using: type[_G]) -> _G: ...
@overload
def from_pandas_adjacency(df: _DataFrame, create_using: None = None) -> Graph[Incomplete]: ...
@_dispatchable
def to_pandas_edgelist(
    G: Graph[_Node],
    source: str | int = "source",
    target: str | int = "target",
    nodelist: Iterable[_Node] | None = None,
    dtype: _ExtensionDtype | None = None,
    edge_key: str | int | None = None,
) -> _DataFrame: ...
@overload
def from_pandas_edgelist(
    df: _DataFrame,
    source: str | int,
    target: str | int,
    edge_attr: str | int | list[str | int] | tuple[str | int] | Literal[True] | None,
    create_using: type[_G],
    edge_key: str | None = None,
) -> _G: ...
@overload
def from_pandas_edgelist(
    df: _DataFrame,
    source: str | int = "source",
    target: str | int = "target",
    edge_attr: str | int | list[str | int] | tuple[str | int] | Literal[True] | None = None,
    *,
    create_using: type[_G],
    edge_key: str | None = None,
) -> _G: ...
@overload
def from_pandas_edgelist(
    df: _DataFrame,
    source: str | int = "source",
    target: str | int = "target",
    edge_attr: str | int | list[str | int] | tuple[str | int] | Literal[True] | None = None,
    create_using: None = None,
    edge_key: str | None = None,
) -> Graph[Incomplete]: ...
@_dispatchable
def to_scipy_sparse_array(G: Graph[_Node], nodelist=None, dtype=None, weight="weight", format="csr"): ...
@_dispatchable
def from_scipy_sparse_array(A, parallel_edges=False, create_using=None, edge_attribute="weight"): ...
@_dispatchable
def to_numpy_array(
    G: Graph[_Node],
    nodelist: Collection[_Node] | None = None,
    dtype: numpy.dtype[Incomplete] | None = None,
    order: numpy._OrderCF = None,
    multigraph_weight: Callable[[list[float]], float] = ...,
    weight: str = "weight",
    nonedge: float = 0.0,
) -> numpy.ndarray[Incomplete, numpy.dtype[Incomplete]]: ...
@overload
def from_numpy_array(
    A: numpy.ndarray[Incomplete, Incomplete], parallel_edges: bool = False, create_using: None = None
) -> Graph[Incomplete]: ...
@overload
def from_numpy_array(A: numpy.ndarray[Incomplete, Incomplete], parallel_edges: bool = False, *, create_using: type[_G]) -> _G: ...
@overload
def from_numpy_array(A: numpy.ndarray[Incomplete, Incomplete], parallel_edges: bool, create_using: type[_G]) -> _G: ...



from _typeshed import Incomplete

from antlr4 import (
    DFA as DFA,
    CommonTokenStream as CommonTokenStream,
    Lexer as Lexer,
    LexerATNSimulator as LexerATNSimulator,
    ParserRuleContext as ParserRuleContext,
    PredictionContextCache as PredictionContextCache,
    TerminalNode as TerminalNode,
)
from antlr4.atn.ATNDeserializer import ATNDeserializer as ATNDeserializer
from antlr4.error.ErrorListener import ErrorListener as ErrorListener
from antlr4.error.Errors import LexerNoViableAltException as LexerNoViableAltException
from antlr4.InputStream import InputStream as InputStream
from antlr4.Parser import Parser as Parser
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token
from antlr4.tree.Tree import ParseTree as ParseTree
from antlr4.tree.Trees import Trees as Trees
from antlr4.xpath.XPathLexer import XPathLexer as XPathLexer

class XPath:
    WILDCARD: str
    NOT: str
    parser: Incomplete
    path: Incomplete
    elements: Incomplete
    def __init__(self, parser: Parser, path: str) -> None: ...
    def split(self, path: str): ...
    def getXPathElement(self, wordToken: Token, anywhere: bool): ...
    @staticmethod
    def findAll(tree: ParseTree, xpath: str, parser: Parser): ...
    def evaluate(self, t: ParseTree): ...

class XPathElement:
    nodeName: Incomplete
    invert: bool
    def __init__(self, nodeName: str) -> None: ...

class XPathRuleAnywhereElement(XPathElement):
    ruleIndex: Incomplete
    def __init__(self, ruleName: str, ruleIndex: int) -> None: ...
    def evaluate(self, t: ParseTree): ...

class XPathRuleElement(XPathElement):
    ruleIndex: Incomplete
    def __init__(self, ruleName: str, ruleIndex: int) -> None: ...
    def evaluate(self, t: ParseTree): ...

class XPathTokenAnywhereElement(XPathElement):
    tokenType: Incomplete
    def __init__(self, ruleName: str, tokenType: int) -> None: ...
    def evaluate(self, t: ParseTree): ...

class XPathTokenElement(XPathElement):
    tokenType: Incomplete
    def __init__(self, ruleName: str, tokenType: int) -> None: ...
    def evaluate(self, t: ParseTree): ...

class XPathWildcardAnywhereElement(XPathElement):
    def __init__(self) -> None: ...
    def evaluate(self, t: ParseTree): ...

class XPathWildcardElement(XPathElement):
    def __init__(self) -> None: ...
    def evaluate(self, t: ParseTree): ...



from typing import Literal

from reportlab.pdfgen.canvas import Canvas
from reportlab.platypus.flowables import Flowable

class Frame:
    id: str | None
    x1: float
    y1: float
    width: float
    height: float
    leftPadding: float
    bottomPadding: float
    rightPadding: float
    topPadding: float
    showBoundary: int
    def __init__(
        self,
        x1: float,
        y1: float,
        width: float,
        height: float,
        leftPadding: float = 6,
        bottomPadding: float = 6,
        rightPadding: float = 6,
        topPadding: float = 6,
        id: str | None = None,
        showBoundary: int = 0,
        overlapAttachedSpace=None,
        _debug=None,
    ) -> None: ...
    def add(self, flowable: Flowable, canv: Canvas, trySplit: int = 0) -> Literal[0, 1]: ...
    def split(self, flowable: Flowable, canv: Canvas) -> list[Flowable]: ...
    def drawBoundary(self, canv: Canvas) -> None: ...
    def addFromList(self, drawlist: list[Flowable], canv: Canvas) -> None: ...
    def add_generated_content(self, *C: Flowable) -> None: ...

__all__ = ("Frame",)



from asyncio.events import AbstractEventLoop
from asyncio.tasks import Task, _TaskCompatibleCoro
from typing import Any, TypeVar

from .context import Context as _Context

_T_co = TypeVar("_T_co", covariant=True)

class AsyncContext(_Context):
    def __init__(
        self, context_missing: str = "LOG_ERROR", loop: AbstractEventLoop | None = None, use_task_factory: bool = True
    ) -> None: ...
    def clear_trace_entities(self) -> None: ...

class TaskLocalStorage:
    def __init__(self, loop: AbstractEventLoop | None = None) -> None: ...
    # Sets unknown items on the current task's context attribute
    def __setattr__(self, name: str, value: Any) -> None: ...
    # Returns unknown items from the current tasks context attribute
    def __getattribute__(self, item: str) -> Any | None: ...
    def clear(self) -> None: ...

def task_factory(loop: AbstractEventLoop | None, coro: _TaskCompatibleCoro[_T_co]) -> Task[_T_co]: ...



from typing import Any

class PersistentSearch:
    connection: Any
    changes_only: Any
    notifications: Any
    message_id: Any
    base: Any
    filter: Any
    scope: Any
    dereference_aliases: Any
    attributes: Any
    size_limit: Any
    time_limit: Any
    controls: Any
    def __init__(
        self,
        connection,
        search_base,
        search_filter,
        search_scope,
        dereference_aliases,
        attributes,
        size_limit,
        time_limit,
        controls,
        changes_only,
        events_type,
        notifications,
        streaming,
        callback,
    ) -> None: ...
    def start(self) -> None: ...
    def stop(self, unbind: bool = True) -> None: ...
    def next(self, block: bool = False, timeout=None): ...
    def funnel(self, block: bool = False, timeout=None) -> None: ...



import re
from collections.abc import Callable
from types import ModuleType

tag_regexp: re.Pattern[str]

def getETreeBuilder(ElementTreeImplementation): ...

getETreeModule: Callable[..., ModuleType]



import datetime
from _typeshed import FileDescriptorOrPath, ReadableBuffer
from collections.abc import Iterable, Mapping
from shlex import _ShlexInstream
from typing import Literal, NamedTuple, NoReturn, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import deprecated

from ..tls import TLSConfig

_T = TypeVar("_T")
_K = TypeVar("_K")
_V = TypeVar("_V")

@type_check_only
class _EnvKWArgs(TypedDict, total=False):
    base_url: str
    tls: TLSConfig

class URLComponents(NamedTuple):
    scheme: str | None
    netloc: str | None
    url: str
    params: str | None
    query: str | None
    fragment: str | None

@deprecated("utils.create_ipam_pool has been removed. Please use a docker.types.IPAMPool object instead.")
def create_ipam_pool(*args, **kwargs) -> NoReturn: ...
@deprecated("utils.create_ipam_config has been removed. Please use a docker.types.IPAMConfig object instead.")
def create_ipam_config(*args, **kwargs) -> NoReturn: ...
def decode_json_header(header: str | ReadableBuffer): ...
def compare_version(v1: str, v2: str) -> Literal[0, -1, 1]: ...
def version_lt(v1: str, v2: str) -> bool: ...
def version_gte(v1: str, v2: str) -> bool: ...
def convert_port_bindings(
    port_bindings: Mapping[str, int | list[int] | tuple[str, int] | None],
) -> dict[str, list[dict[str, str]]]: ...
@overload
def convert_volume_binds(binds: list[_T]) -> list[_T]: ...
@overload
def convert_volume_binds(binds: Mapping[str | bytes, bytes | str | dict[str, bytes | str]]) -> list[str]: ...
@overload
def convert_tmpfs_mounts(tmpfs: dict[_K, _V]) -> dict[_K, _V]: ...
@overload
def convert_tmpfs_mounts(tmpfs: list[str]) -> dict[str, str]: ...
@overload
def convert_service_networks(networks: None) -> None: ...
@overload
def convert_service_networks(networks: list[str] | list[dict[str, str]] | list[str | dict[str, str]]) -> list[dict[str, str]]: ...
def parse_repository_tag(repo_name: str) -> tuple[str, str | None]: ...
@overload
def parse_host(addr: None, is_win32: Literal[True], tls: bool = False) -> Literal["npipe:////./pipe/docker_engine"]: ...
@overload
def parse_host(
    addr: None, is_win32: Literal[False] = False, tls: bool = False
) -> Literal["http+unix:///var/run/docker.sock"]: ...
@overload
def parse_host(addr: str | None, is_win32: bool = False, tls: bool = False) -> str | bytes: ...
def parse_devices(devices: Iterable[str | dict[str, str]]) -> list[dict[str, str]]: ...
def kwargs_from_env(environment: Mapping[str, str] | None = None) -> _EnvKWArgs: ...
def convert_filters(filters) -> str: ...
def datetime_to_timestamp(dt: datetime.datetime) -> int: ...
def parse_bytes(s: float | str) -> float: ...
def normalize_links(links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]]): ...
def parse_env_file(env_file: FileDescriptorOrPath) -> dict[str, str]: ...
def split_command(command: str | _ShlexInstream) -> list[str]: ...
def format_environment(environment: Mapping[str, object | None]) -> list[str]: ...
def format_extra_hosts(
    extra_hosts: Mapping[object, object], task: bool = False  # keys and values are converted to str
) -> list[str]: ...
@deprecated("utils.create_host_config has been removed. Please use a docker.types.HostConfig object instead.")
def create_host_config(self, *args, **kwargs) -> NoReturn: ...



from _typeshed import Incomplete

from reportlab.platypus.flowables import Flowable

class Barcode(Flowable):
    fontName: str
    fontSize: int
    humanReadable: int
    value: Incomplete
    gap: Incomplete
    def __init__(self, value: str = "", **kwd) -> None: ...
    valid: int
    validated: Incomplete
    def validate(self) -> None: ...
    encoded: Incomplete
    def encode(self) -> None: ...
    decomposed: Incomplete
    def decompose(self) -> None: ...
    barHeight: Incomplete
    def computeSize(self, *args) -> None: ...
    @property
    def width(self): ...
    @width.setter
    def width(self, v) -> None: ...
    @property
    def height(self): ...
    @height.setter
    def height(self, v) -> None: ...
    def draw(self) -> None: ...
    def drawHumanReadable(self) -> None: ...
    def rect(self, x, y, w, h) -> None: ...
    def annotate(self, x, y, text, fontName, fontSize, anchor: str = "middle") -> None: ...

class MultiWidthBarcode(Barcode):
    barHeight: Incomplete
    def computeSize(self, *args) -> None: ...
    def draw(self) -> None: ...

class I2of5(Barcode):
    patterns: Incomplete
    barHeight: Incomplete
    barWidth: Incomplete
    ratio: float
    checksum: int
    bearers: float
    bearerBox: bool
    quiet: int
    lquiet: Incomplete
    rquiet: Incomplete
    stop: int
    def __init__(self, value: str = "", **args) -> None: ...
    valid: int
    validated: Incomplete
    def validate(self): ...
    encoded: Incomplete
    def encode(self) -> None: ...
    decomposed: Incomplete
    def decompose(self): ...

class MSI(Barcode):
    patterns: Incomplete
    stop: int
    barHeight: Incomplete
    barWidth: Incomplete
    ratio: float
    checksum: int
    bearers: float
    quiet: int
    lquiet: Incomplete
    rquiet: Incomplete
    def __init__(self, value: str = "", **args) -> None: ...
    valid: int
    validated: Incomplete
    def validate(self): ...
    encoded: Incomplete
    def encode(self) -> None: ...
    decomposed: Incomplete
    def decompose(self): ...

class Codabar(Barcode):
    patterns: Incomplete
    values: Incomplete
    chars: Incomplete
    stop: int
    barHeight: Incomplete
    barWidth: Incomplete
    ratio: float
    checksum: int
    bearers: float
    quiet: int
    lquiet: Incomplete
    rquiet: Incomplete
    def __init__(self, value: str = "", **args) -> None: ...
    valid: int
    Valid: int
    validated: Incomplete
    def validate(self): ...
    encoded: Incomplete
    def encode(self) -> None: ...
    decomposed: Incomplete
    def decompose(self): ...

class Code11(Barcode):
    chars: str
    patterns: Incomplete
    values: Incomplete
    stop: int
    barHeight: Incomplete
    barWidth: Incomplete
    ratio: float
    checksum: int
    bearers: float
    quiet: int
    lquiet: Incomplete
    rquiet: Incomplete
    def __init__(self, value: str = "", **args) -> None: ...
    valid: int
    Valid: int
    validated: Incomplete
    def validate(self): ...
    encoded: Incomplete
    def encode(self) -> None: ...
    decomposed: Incomplete
    def decompose(self): ...



from typing import ClassVar

from .geometry import Point2D, PositiveSize2D, Transform2D

class XDRPoint2D(Point2D):
    namespace: ClassVar[None]  # type: ignore[assignment]
    # Same as parent
    # x = Point2D.x
    # y = Point2D.y

class XDRPositiveSize2D(PositiveSize2D):
    namespace: ClassVar[None]  # type: ignore[assignment]
    # Same as parent
    # cx = PositiveSize2D.cx
    # cy = PositiveSize2D.cy

class XDRTransform2D(Transform2D):
    namespace: ClassVar[None]  # type: ignore[assignment]
    # Same as parent
    # rot = Transform2D.rot
    # flipH = Transform2D.flipH
    # flipV = Transform2D.flipV
    # off = Transform2D.off
    # ext = Transform2D.ext
    # chOff = Transform2D.chOff
    # chExt = Transform2D.chExt



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["spectral_bipartivity"]

@_dispatchable
def spectral_bipartivity(G: Graph[_Node], nodes=None, weight: str = "weight") -> float | dict[Incomplete, Incomplete]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_regular", "is_k_regular", "k_factor"]

@_dispatchable
def is_regular(G: Graph[_Node]) -> bool: ...
@_dispatchable
def is_k_regular(G: Graph[_Node], k) -> bool: ...
@_dispatchable
def k_factor(G: Graph[_Node], k, matching_weight: str | None = "weight"): ...



from hvac.api.vault_api_base import VaultApiBase

class JWT(VaultApiBase):
    DEFAULT_PATH: str
    def resolve_path(self, path): ...
    def configure(
        self,
        oidc_discovery_url=None,
        oidc_discovery_ca_pem=None,
        oidc_client_id=None,
        oidc_client_secret=None,
        oidc_response_mode=None,
        oidc_response_types=None,
        jwks_url=None,
        jwks_ca_pem=None,
        jwt_validation_pubkeys=None,
        bound_issuer=None,
        jwt_supported_algs=None,
        default_role=None,
        provider_config=None,
        path: str | None = None,
        namespace_in_state: bool | None = None,
    ): ...
    def read_config(self, path=None): ...
    def create_role(
        self,
        name,
        user_claim,
        allowed_redirect_uris,
        role_type: str = "jwt",
        bound_audiences=None,
        clock_skew_leeway=None,
        expiration_leeway=None,
        not_before_leeway=None,
        bound_subject=None,
        bound_claims=None,
        groups_claim=None,
        claim_mappings=None,
        oidc_scopes=None,
        bound_claims_type: str = "string",
        verbose_oidc_logging: bool = False,
        token_ttl=None,
        token_max_ttl=None,
        token_policies=None,
        token_bound_cidrs=None,
        token_explicit_max_ttl=None,
        token_no_default_policy=None,
        token_num_uses=None,
        token_period=None,
        token_type=None,
        path=None,
        user_claim_json_pointer=None,
    ): ...
    def read_role(self, name, path=None): ...
    def list_roles(self, path=None): ...
    def delete_role(self, name, path=None): ...
    def oidc_authorization_url_request(self, role, redirect_uri, path=None): ...
    def oidc_callback(self, state, nonce, code, path=None): ...
    def jwt_login(self, role, jwt, use_token: bool = True, path=None): ...



from _typeshed import Incomplete

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["trophic_levels", "trophic_differences", "trophic_incoherence_parameter"]

@_dispatchable
def trophic_levels(G: DiGraph[_Node], weight="weight") -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def trophic_differences(G: DiGraph[_Node], weight="weight") -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def trophic_incoherence_parameter(G: DiGraph[_Node], weight="weight", cannibalism: bool = False) -> float: ...



from email.message import Message
from http.client import HTTPResponse
from typing import Any, Final

class Response:
    def __init__(self, response: HTTPResponse) -> None: ...
    @property
    def status_code(self) -> int: ...
    @property
    def body(self) -> bytes: ...
    @property
    def headers(self) -> Message: ...
    @property
    def to_dict(self) -> dict[str, Any] | None: ...  # dict of response from API if body is not empty

class Client:
    methods: Final[set[str]]
    host: str
    request_headers: dict[str, str]
    append_slash: bool
    timeout: int
    def __init__(
        self,
        host: str,
        request_headers: dict[str, str] | None = None,
        version: int | None = None,
        url_path: list[str] | None = None,
        append_slash: bool = False,
        timeout: int | None = None,
    ) -> None: ...
    def _(self, name: str) -> Client: ...
    def __getattr__(self, name: str) -> Client | Response: ...



from typing import Any

class OAuth1Error(Exception):
    error: Any
    description: str
    uri: Any
    status_code: Any
    def __init__(self, description=None, uri=None, status_code: int = 400, request=None) -> None: ...
    def in_uri(self, uri): ...
    @property
    def twotuples(self): ...
    @property
    def urlencoded(self): ...

class InsecureTransportError(OAuth1Error):
    error: str
    description: str

class InvalidSignatureMethodError(OAuth1Error):
    error: str

class InvalidRequestError(OAuth1Error):
    error: str

class InvalidClientError(OAuth1Error):
    error: str



from typing import Final

XK_BackSpace: Final = 0xFF08
XK_Tab: Final = 0xFF09
XK_Linefeed: Final = 0xFF0A
XK_Clear: Final = 0xFF0B
XK_Return: Final = 0xFF0D
XK_Pause: Final = 0xFF13
XK_Scroll_Lock: Final = 0xFF14
XK_Sys_Req: Final = 0xFF15
XK_Escape: Final = 0xFF1B
XK_Delete: Final = 0xFFFF
XK_Multi_key: Final = 0xFF20
XK_SingleCandidate: Final = 0xFF3C
XK_MultipleCandidate: Final = 0xFF3D
XK_PreviousCandidate: Final = 0xFF3E
XK_Kanji: Final = 0xFF21
XK_Muhenkan: Final = 0xFF22
XK_Henkan_Mode: Final = 0xFF23
XK_Henkan: Final = 0xFF23
XK_Romaji: Final = 0xFF24
XK_Hiragana: Final = 0xFF25
XK_Katakana: Final = 0xFF26
XK_Hiragana_Katakana: Final = 0xFF27
XK_Zenkaku: Final = 0xFF28
XK_Hankaku: Final = 0xFF29
XK_Zenkaku_Hankaku: Final = 0xFF2A
XK_Touroku: Final = 0xFF2B
XK_Massyo: Final = 0xFF2C
XK_Kana_Lock: Final = 0xFF2D
XK_Kana_Shift: Final = 0xFF2E
XK_Eisu_Shift: Final = 0xFF2F
XK_Eisu_toggle: Final = 0xFF30
XK_Zen_Koho: Final = 0xFF3D
XK_Mae_Koho: Final = 0xFF3E
XK_Home: Final = 0xFF50
XK_Left: Final = 0xFF51
XK_Up: Final = 0xFF52
XK_Right: Final = 0xFF53
XK_Down: Final = 0xFF54
XK_Prior: Final = 0xFF55
XK_Page_Up: Final = 0xFF55
XK_Next: Final = 0xFF56
XK_Page_Down: Final = 0xFF56
XK_End: Final = 0xFF57
XK_Begin: Final = 0xFF58
XK_Select: Final = 0xFF60
XK_Print: Final = 0xFF61
XK_Execute: Final = 0xFF62
XK_Insert: Final = 0xFF63
XK_Undo: Final = 0xFF65
XK_Redo: Final = 0xFF66
XK_Menu: Final = 0xFF67
XK_Find: Final = 0xFF68
XK_Cancel: Final = 0xFF69
XK_Help: Final = 0xFF6A
XK_Break: Final = 0xFF6B
XK_Mode_switch: Final = 0xFF7E
XK_script_switch: Final = 0xFF7E
XK_Num_Lock: Final = 0xFF7F
XK_KP_Space: Final = 0xFF80
XK_KP_Tab: Final = 0xFF89
XK_KP_Enter: Final = 0xFF8D
XK_KP_F1: Final = 0xFF91
XK_KP_F2: Final = 0xFF92
XK_KP_F3: Final = 0xFF93
XK_KP_F4: Final = 0xFF94
XK_KP_Home: Final = 0xFF95
XK_KP_Left: Final = 0xFF96
XK_KP_Up: Final = 0xFF97
XK_KP_Right: Final = 0xFF98
XK_KP_Down: Final = 0xFF99
XK_KP_Prior: Final = 0xFF9A
XK_KP_Page_Up: Final = 0xFF9A
XK_KP_Next: Final = 0xFF9B
XK_KP_Page_Down: Final = 0xFF9B
XK_KP_End: Final = 0xFF9C
XK_KP_Begin: Final = 0xFF9D
XK_KP_Insert: Final = 0xFF9E
XK_KP_Delete: Final = 0xFF9F
XK_KP_Equal: Final = 0xFFBD
XK_KP_Multiply: Final = 0xFFAA
XK_KP_Add: Final = 0xFFAB
XK_KP_Separator: Final = 0xFFAC
XK_KP_Subtract: Final = 0xFFAD
XK_KP_Decimal: Final = 0xFFAE
XK_KP_Divide: Final = 0xFFAF
XK_KP_0: Final = 0xFFB0
XK_KP_1: Final = 0xFFB1
XK_KP_2: Final = 0xFFB2
XK_KP_3: Final = 0xFFB3
XK_KP_4: Final = 0xFFB4
XK_KP_5: Final = 0xFFB5
XK_KP_6: Final = 0xFFB6
XK_KP_7: Final = 0xFFB7
XK_KP_8: Final = 0xFFB8
XK_KP_9: Final = 0xFFB9
XK_F1: Final = 0xFFBE
XK_F2: Final = 0xFFBF
XK_F3: Final = 0xFFC0
XK_F4: Final = 0xFFC1
XK_F5: Final = 0xFFC2
XK_F6: Final = 0xFFC3
XK_F7: Final = 0xFFC4
XK_F8: Final = 0xFFC5
XK_F9: Final = 0xFFC6
XK_F10: Final = 0xFFC7
XK_F11: Final = 0xFFC8
XK_L1: Final = 0xFFC8
XK_F12: Final = 0xFFC9
XK_L2: Final = 0xFFC9
XK_F13: Final = 0xFFCA
XK_L3: Final = 0xFFCA
XK_F14: Final = 0xFFCB
XK_L4: Final = 0xFFCB
XK_F15: Final = 0xFFCC
XK_L5: Final = 0xFFCC
XK_F16: Final = 0xFFCD
XK_L6: Final = 0xFFCD
XK_F17: Final = 0xFFCE
XK_L7: Final = 0xFFCE
XK_F18: Final = 0xFFCF
XK_L8: Final = 0xFFCF
XK_F19: Final = 0xFFD0
XK_L9: Final = 0xFFD0
XK_F20: Final = 0xFFD1
XK_L10: Final = 0xFFD1
XK_F21: Final = 0xFFD2
XK_R1: Final = 0xFFD2
XK_F22: Final = 0xFFD3
XK_R2: Final = 0xFFD3
XK_F23: Final = 0xFFD4
XK_R3: Final = 0xFFD4
XK_F24: Final = 0xFFD5
XK_R4: Final = 0xFFD5
XK_F25: Final = 0xFFD6
XK_R5: Final = 0xFFD6
XK_F26: Final = 0xFFD7
XK_R6: Final = 0xFFD7
XK_F27: Final = 0xFFD8
XK_R7: Final = 0xFFD8
XK_F28: Final = 0xFFD9
XK_R8: Final = 0xFFD9
XK_F29: Final = 0xFFDA
XK_R9: Final = 0xFFDA
XK_F30: Final = 0xFFDB
XK_R10: Final = 0xFFDB
XK_F31: Final = 0xFFDC
XK_R11: Final = 0xFFDC
XK_F32: Final = 0xFFDD
XK_R12: Final = 0xFFDD
XK_F33: Final = 0xFFDE
XK_R13: Final = 0xFFDE
XK_F34: Final = 0xFFDF
XK_R14: Final = 0xFFDF
XK_F35: Final = 0xFFE0
XK_R15: Final = 0xFFE0
XK_Shift_L: Final = 0xFFE1
XK_Shift_R: Final = 0xFFE2
XK_Control_L: Final = 0xFFE3
XK_Control_R: Final = 0xFFE4
XK_Caps_Lock: Final = 0xFFE5
XK_Shift_Lock: Final = 0xFFE6
XK_Meta_L: Final = 0xFFE7
XK_Meta_R: Final = 0xFFE8
XK_Alt_L: Final = 0xFFE9
XK_Alt_R: Final = 0xFFEA
XK_Super_L: Final = 0xFFEB
XK_Super_R: Final = 0xFFEC
XK_Hyper_L: Final = 0xFFED
XK_Hyper_R: Final = 0xFFEE



import datetime
from collections.abc import Callable, Generator, Iterable, Iterator, Mapping, Sequence
from typing import Final, Literal, overload
from typing_extensions import Self, TypeAlias

from dateutil.parser._parser import _TzInfos

from ._common import weekday as weekdaybase

__all__ = [
    "rrule",
    "rruleset",
    "rrulestr",
    "YEARLY",
    "MONTHLY",
    "WEEKLY",
    "DAILY",
    "HOURLY",
    "MINUTELY",
    "SECONDLY",
    "MO",
    "TU",
    "WE",
    "TH",
    "FR",
    "SA",
    "SU",
]

M366MASK: Final[tuple[int, ...]]
MDAY366MASK: Final[tuple[int, ...]]
MDAY365MASK: Final[tuple[int, ...]]
NMDAY366MASK: Final[tuple[int, ...]]
NMDAY365MASK: Final[list[int]]
M366RANGE: Final[tuple[int, ...]]
M365RANGE: Final[tuple[int, ...]]
WDAYMASK: Final[list[int]]
M365MASK: Final[tuple[int, ...]]
FREQNAMES: Final[list[str]]
YEARLY: Final = 0
MONTHLY: Final = 1
WEEKLY: Final = 2
DAILY: Final = 3
HOURLY: Final = 4
MINUTELY: Final = 5
SECONDLY: Final = 6

class weekday(weekdaybase): ...

weekdays: tuple[weekday, weekday, weekday, weekday, weekday, weekday, weekday]
MO: weekday
TU: weekday
WE: weekday
TH: weekday
FR: weekday
SA: weekday
SU: weekday

class rrulebase:
    def __init__(self, cache: bool | None = False) -> None: ...
    def __iter__(self) -> Iterator[datetime.datetime]: ...
    def __getitem__(self, item: int | slice) -> datetime.datetime: ...
    def __contains__(self, item: datetime.datetime) -> bool: ...
    def count(self) -> int | None: ...
    def before(self, dt: datetime.datetime, inc: bool = False): ...
    def after(self, dt: datetime.datetime, inc: bool = False): ...
    def xafter(self, dt: datetime.datetime, count: int | None = None, inc: bool = False) -> Generator[datetime.datetime]: ...
    def between(
        self, after: datetime.datetime, before: datetime.datetime, inc: bool = False, count: int = 1
    ) -> list[datetime.datetime]: ...

class rrule(rrulebase):
    def __init__(
        self,
        freq: Literal[0, 1, 2, 3, 4, 5, 6],
        dtstart: datetime.date | None = None,
        interval: int = 1,
        wkst: weekday | int | None = None,
        count: int | None = None,
        until: datetime.date | int | None = None,
        bysetpos: int | Iterable[int] | None = None,
        bymonth: int | Iterable[int] | None = None,
        bymonthday: int | Iterable[int] | None = None,
        byyearday: int | Iterable[int] | None = None,
        byeaster: int | Iterable[int] | None = None,
        byweekno: int | Iterable[int] | None = None,
        byweekday: int | weekday | Iterable[int] | Iterable[weekday] | None = None,
        byhour: int | Iterable[int] | None = None,
        byminute: int | Iterable[int] | None = None,
        bysecond: int | Iterable[int] | None = None,
        cache: bool | None = False,
    ) -> None: ...
    def replace(
        self,
        *,
        freq: Literal[0, 1, 2, 3, 4, 5, 6] = ...,
        dtstart: datetime.date | None = ...,
        interval: int = ...,
        wkst: weekday | int | None = ...,
        count: int | None = ...,
        until: datetime.date | int | None = ...,
        bysetpos: int | Iterable[int] | None = None,
        bymonth: int | Iterable[int] | None = None,
        bymonthday: int | Iterable[int] | None = None,
        byyearday: int | Iterable[int] | None = None,
        byeaster: int | Iterable[int] | None = None,
        byweekno: int | Iterable[int] | None = None,
        byweekday: int | weekday | Iterable[int] | Iterable[weekday] | None = None,
        byhour: int | Iterable[int] | None = None,
        byminute: int | Iterable[int] | None = None,
        bysecond: int | Iterable[int] | None = None,
        cache: bool | None = ...,
    ) -> Self: ...

_RRule: TypeAlias = rrule

class _iterinfo:
    __slots__ = [
        "rrule",
        "lastyear",
        "lastmonth",
        "yearlen",
        "nextyearlen",
        "yearordinal",
        "yearweekday",
        "mmask",
        "mrange",
        "mdaymask",
        "nmdaymask",
        "wdaymask",
        "wnomask",
        "nwdaymask",
        "eastermask",
    ]
    rrule: _RRule
    def __init__(self, rrule: _RRule) -> None: ...
    yearlen: int | None
    nextyearlen: int | None
    yearordinal: int | None
    yearweekday: int | None
    mmask: Sequence[int] | None
    mdaymask: Sequence[int] | None
    nmdaymask: Sequence[int] | None
    wdaymask: Sequence[int] | None
    mrange: Sequence[int] | None
    wnomask: Sequence[int] | None
    nwdaymask: Sequence[int] | None
    eastermask: Sequence[int] | None
    lastyear: int | None
    lastmonth: int | None
    def rebuild(self, year: int, month: int) -> None: ...
    def ydayset(self, year: int, month: int, day: int) -> tuple[Iterable[int | None], int, int]: ...
    def mdayset(self, year: int, month: int, day: int) -> tuple[Iterable[int | None], int, int]: ...
    def wdayset(self, year: int, month: int, day: int) -> tuple[Iterable[int | None], int, int]: ...
    def ddayset(self, year: int, month: int, day: int) -> tuple[Iterable[int | None], int, int]: ...
    def htimeset(self, hour: int, minute: int, second: int) -> list[datetime.time]: ...
    def mtimeset(self, hour: int, minute: int, second: int) -> list[datetime.time]: ...
    def stimeset(self, hour: int, minute: int, second: int) -> tuple[datetime.time, ...]: ...

class rruleset(rrulebase):
    class _genitem:
        dt: datetime.datetime
        genlist: list[Self]
        gen: Iterator[datetime.datetime]
        def __init__(self, genlist: list[Self], gen: Iterator[datetime.datetime]) -> None: ...
        def __next__(self) -> None: ...
        next = __next__
        def __lt__(self, other: Self) -> bool: ...
        def __gt__(self, other: Self) -> bool: ...
        def __eq__(self, other: object) -> bool: ...
        def __ne__(self, other: object) -> bool: ...

    def __init__(self, cache: bool | None = False) -> None: ...
    def rrule(self, rrule: _RRule) -> None: ...
    def rdate(self, rdate: datetime.datetime) -> None: ...
    def exrule(self, exrule: _RRule) -> None: ...
    def exdate(self, exdate: datetime.datetime) -> None: ...

class _rrulestr:
    @overload
    def __call__(
        self,
        s: str,
        *,
        forceset: Literal[True],
        dtstart: datetime.date | None = None,
        cache: bool | None = None,
        unfold: bool = False,
        compatible: bool = False,
        ignoretz: bool = False,
        tzids: Callable[[str], datetime.tzinfo] | Mapping[str, datetime.tzinfo] | None = None,
        tzinfos: _TzInfos | None = None,
    ) -> rruleset: ...
    @overload
    def __call__(
        self,
        s: str,
        *,
        compatible: Literal[True],
        dtstart: datetime.date | None = None,
        cache: bool | None = None,
        unfold: bool = False,
        forceset: bool = False,
        ignoretz: bool = False,
        tzids: Callable[[str], datetime.tzinfo] | Mapping[str, datetime.tzinfo] | None = None,
        tzinfos: _TzInfos | None = None,
    ) -> rruleset: ...
    @overload
    def __call__(
        self,
        s: str,
        *,
        dtstart: datetime.date | None = None,
        cache: bool | None = False,
        unfold: bool = False,
        forceset: bool = False,
        compatible: bool = False,
        ignoretz: bool = False,
        tzids: Callable[[str], datetime.tzinfo] | Mapping[str, datetime.tzinfo] | None = None,
        tzinfos: _TzInfos | None = None,
    ) -> rrule | rruleset: ...

rrulestr: _rrulestr



from _typeshed import Incomplete
from typing import Any, Literal, TypeVar

from antlr4.ParserRuleContext import ParserRuleContext, RuleContext
from antlr4.Token import Token as Token

INVALID_INTERVAL: Incomplete

class Tree: ...
class SyntaxTree(Tree): ...
class ParseTree(SyntaxTree): ...
class RuleNode(ParseTree): ...
class TerminalNode(ParseTree): ...
class ErrorNode(TerminalNode): ...

_GenericType = TypeVar("_GenericType", bound=type)

class ParseTreeVisitor:
    def visit(self, tree: Tree) -> None: ...
    def visitChildren(self, node) -> None: ...
    def visitTerminal(self, node: TerminalNode) -> None: ...
    def visitErrorNode(self, node: ErrorNode) -> None: ...
    def defaultResult(self) -> None: ...
    def aggregateResult(self, aggregate, nextResult: _GenericType) -> _GenericType: ...
    def shouldVisitNextChild(self, node, currentResult) -> Literal[True]: ...

class ParseTreeListener:
    def visitTerminal(self, node: TerminalNode) -> None: ...
    def visitErrorNode(self, node: ErrorNode) -> None: ...
    def enterEveryRule(self, ctx: ParserRuleContext) -> None: ...
    def exitEveryRule(self, ctx: ParserRuleContext) -> None: ...

class TerminalNodeImpl(TerminalNode):
    __slots__ = ("parentCtx", "symbol")
    parentCtx: RuleContext | None
    symbol: Token
    def __init__(self, symbol: Token) -> None: ...
    def __setattr__(self, key: str, value: Any) -> None: ...
    def getChild(self, i: int) -> None: ...
    def getSymbol(self) -> Token: ...
    def getParent(self) -> RuleContext | None: ...
    def getPayload(self) -> Token: ...
    def getSourceInterval(self) -> tuple[Literal[-1], Literal[-2]] | tuple[int | None, int | None]: ...
    def getChildCount(self) -> Literal[0]: ...
    def accept(self, visitor: ParseTreeVisitor) -> None: ...
    def getText(self) -> str: ...

class ErrorNodeImpl(TerminalNodeImpl, ErrorNode):
    def __init__(self, token: Token) -> None: ...
    def accept(self, visitor: ParseTreeVisitor) -> None: ...

class ParseTreeWalker:
    DEFAULT: ParseTreeWalker
    def walk(self, listener: ParseTreeListener, t: ParseTree) -> None: ...
    def enterRule(self, listener: ParseTreeListener, r: RuleNode) -> None: ...
    def exitRule(self, listener: ParseTreeListener, r: RuleNode) -> None: ...



"""
A package to compare python-style regexes and test if they have intersections.
Based on the `greenery`-package by @qntm, adapted and specialized for `lark-parser`
"""

from typing import Iterable, Tuple

from interegular.fsm import FSM
from interegular.patterns import Pattern, parse_pattern, REFlags, Unsupported, InvalidSyntax
from interegular.comparator import Comparator
from interegular.utils import logger

__all__ = ['FSM', 'Pattern', 'Comparator', 'parse_pattern', 'compare_patterns', 'compare_regexes', '__version__', 'REFlags', 'Unsupported',
           'InvalidSyntax']


def compare_regexes(*regexes: str) -> Iterable[Tuple[str, str]]:
    """
    Checks the regexes for intersections. Returns all pairs it found
    """
    c = Comparator({r: parse_pattern(r) for r in regexes})
    print(c._patterns)
    return c.check(regexes)


def compare_patterns(*ps: Pattern) -> Iterable[Tuple[Pattern, Pattern]]:
    """
    Checks the Patterns for intersections. Returns all pairs it found
    """
    c = Comparator({p: p for p in ps})
    return c.check(ps)


__version__ = "0.3.3"



from collections import OrderedDict
from typing import Any, ClassVar

from django.db import models
from django.forms import Form
from django_filters import filterset
from django_filters.filters import Filter

# REST framework field mappings support all Django field types
FILTER_FOR_DBFIELD_DEFAULTS: dict[type[models.Field[Any, Any]], dict[str, Any]]

class FilterSet(filterset.FilterSet):
    FILTER_DEFAULTS: ClassVar[dict[type[models.Field[Any, Any]], dict[str, Any]]] = ...  # DRF field mappings
    base_filters: OrderedDict[str, Filter]
    declared_filters: OrderedDict[str, Filter]
    @property
    def form(self) -> Form: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.attr_value_pb2
import tensorflow.core.framework.full_type_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class NodeDef(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttrEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue: ...
        def __init__(
            self, *, key: builtins.str | None = ..., value: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class ExperimentalDebugInfo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ORIGINAL_NODE_NAMES_FIELD_NUMBER: builtins.int
        ORIGINAL_FUNC_NAMES_FIELD_NUMBER: builtins.int
        @property
        def original_node_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
            """Opaque string inserted into error messages created by the runtime.

            This is intended to store the list of names of the nodes from the
            original graph that this node was derived. For example if this node, say
            C, was result of a fusion of 2 nodes A and B, then 'original_node' would
            be {A, B}. This information can be used to map errors originating at the
            current node to some top level source code.
            """

        @property
        def original_func_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
            """This is intended to store the list of names of the functions from the
            original graph that this node was derived. For example if this node, say
            C, was result of a fusion of node A in function FA and node B in function
            FB, then `original_funcs` would be {FA, FB}. If the node is in the top
            level graph, the `original_func` is empty. This information, with the
            `original_node_names` can be used to map errors originating at the
            current ndoe to some top level source code.
            """

        def __init__(
            self,
            *,
            original_node_names: collections.abc.Iterable[builtins.str] | None = ...,
            original_func_names: collections.abc.Iterable[builtins.str] | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "original_func_names", b"original_func_names", "original_node_names", b"original_node_names"
            ],
        ) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    OP_FIELD_NUMBER: builtins.int
    INPUT_FIELD_NUMBER: builtins.int
    DEVICE_FIELD_NUMBER: builtins.int
    ATTR_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_DEBUG_INFO_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name given to this operator. Used for naming inputs,
    logging, visualization, etc.  Unique within a single GraphDef.
    Must match the regexp "[A-Za-z0-9.][A-Za-z0-9_>./]*".
    """
    op: builtins.str
    """The operation name.  There may be custom parameters in attrs.
    Op names starting with an underscore are reserved for internal use.
    """
    device: builtins.str
    """A (possibly partial) specification for the device on which this
    node should be placed.
    The expected syntax for this string is as follows:

    DEVICE_SPEC ::= PARTIAL_SPEC

    PARTIAL_SPEC ::= ("/" CONSTRAINT) *
    CONSTRAINT ::= ("job:" JOB_NAME)
                 | ("replica:" [1-9][0-9]*)
                 | ("task:" [1-9][0-9]*)
                 | ("device:" [A-Za-z]* ":" ([1-9][0-9]* | "*") )

    Valid values for this string include:
    * "/job:worker/replica:0/task:1/device:GPU:3"  (full specification)
    * "/job:worker/device:GPU:3"                   (partial specification)
    * ""                                    (no specification)

    If the constraints do not resolve to a single device (or if this
    field is empty or not present), the runtime will attempt to
    choose a device automatically.
    """
    @property
    def input(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Each input is "node:src_output" with "node" being a string name and
        "src_output" indicating which output tensor to use from "node". If
        "src_output" is 0 the ":0" suffix can be omitted.  Regular inputs
        may optionally be followed by control inputs that have the format
        "^node".
        """

    @property
    def attr(
        self,
    ) -> google.protobuf.internal.containers.MessageMap[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue]:
        """Operation-specific graph-construction-time configuration.
        Note that this should include all attrs defined in the
        corresponding OpDef, including those with a value matching
        the default -- this allows the default to change and makes
        NodeDefs easier to interpret on their own.  However, if
        an attr with a default is not specified in this list, the
        default will be used.
        The "names" (keys) must match the regexp "[a-z][a-z0-9_]+" (and
        one of the names from the corresponding OpDef's attr field).
        The values must have a type matching the corresponding OpDef
        attr's type field.
        TODO(josh11b): Add some examples here showing best practices.
        """

    @property
    def experimental_debug_info(self) -> global___NodeDef.ExperimentalDebugInfo:
        """This stores debug information associated with the node."""

    @property
    def experimental_type(self) -> tensorflow.core.framework.full_type_pb2.FullTypeDef:
        """The complete type of this node. Experimental and subject to change.
        Currently, the field only contains the return types of the node. That will
        extend in the future to contain the entire signature of the node, as a
        function type.
        """

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        op: builtins.str | None = ...,
        input: collections.abc.Iterable[builtins.str] | None = ...,
        device: builtins.str | None = ...,
        attr: collections.abc.Mapping[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue] | None = ...,
        experimental_debug_info: global___NodeDef.ExperimentalDebugInfo | None = ...,
        experimental_type: tensorflow.core.framework.full_type_pb2.FullTypeDef | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "experimental_debug_info", b"experimental_debug_info", "experimental_type", b"experimental_type"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "attr",
            b"attr",
            "device",
            b"device",
            "experimental_debug_info",
            b"experimental_debug_info",
            "experimental_type",
            b"experimental_type",
            "input",
            b"input",
            "name",
            b"name",
            "op",
            b"op",
        ],
    ) -> None: ...

global___NodeDef = NodeDef



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2016 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class MemmappedFileSystemDirectoryElement(google.protobuf.message.Message):
    """A message that describes one region of memmapped file."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OFFSET_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    LENGTH_FIELD_NUMBER: builtins.int
    offset: builtins.int
    name: builtins.str
    length: builtins.int
    def __init__(
        self, *, offset: builtins.int | None = ..., name: builtins.str | None = ..., length: builtins.int | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["length", b"length", "name", b"name", "offset", b"offset"]) -> None: ...

global___MemmappedFileSystemDirectoryElement = MemmappedFileSystemDirectoryElement

@typing.final
class MemmappedFileSystemDirectory(google.protobuf.message.Message):
    """A directory of regions in a memmapped file."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENT_FIELD_NUMBER: builtins.int
    @property
    def element(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MemmappedFileSystemDirectoryElement]: ...
    def __init__(
        self, *, element: collections.abc.Iterable[global___MemmappedFileSystemDirectoryElement] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["element", b"element"]) -> None: ...

global___MemmappedFileSystemDirectory = MemmappedFileSystemDirectory



from typing import Final, final
from typing_extensions import Self

__all__ = ("MAX", "MIN", "InvalidRomanNumeralError", "OutOfRangeError", "RomanNumeral")

MIN: Final = 1
MAX: Final = 4_999

@final
class OutOfRangeError(TypeError): ...

@final
class InvalidRomanNumeralError(ValueError):
    def __init__(self, value: str, *args: object) -> None: ...

@final
class RomanNumeral:
    __slots__ = ("_value",)
    def __init__(self, value: int, /) -> None: ...
    def __int__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __setattr__(self, key: str, value: object) -> None: ...
    def to_uppercase(self) -> str: ...
    def to_lowercase(self) -> str: ...
    @classmethod
    def from_string(cls, string: str, /) -> Self: ...

_ROMAN_NUMERAL_PREFIXES: Final[list[tuple[int, str, str]]]



from _typeshed import Incomplete
from abc import ABC
from collections.abc import Iterable
from typing import Final, Literal

from .drawing import DeviceCMYK, DeviceGray, DeviceRGB
from .fpdf import FPDF
from .syntax import Name, PDFObject

class Pattern(PDFObject):
    type: Name
    pattern_type: int
    def __init__(self, shading: LinearGradient | RadialGradient) -> None: ...
    @property
    def shading(self) -> str: ...

class Type2Function(PDFObject):
    function_type: Final = 2
    domain: str
    c0: str
    c1: str
    n: int
    def __init__(self, color_1, color_2) -> None: ...

class Type3Function(PDFObject):
    function_type: Final = 3
    domain: str
    bounds: str
    encode: str
    n: int

    def __init__(self, functions: Iterable[Incomplete], bounds: Iterable[Incomplete]) -> None: ...
    @property
    def functions(self) -> str: ...

class Shading(PDFObject):
    shading_type: Literal[2, 3]
    background: str | None
    color_space: Name
    coords: list[int]
    function: str
    extend: str
    def __init__(
        self,
        shading_type: Literal[2, 3],
        background: DeviceRGB | DeviceGray | DeviceCMYK | None,
        color_space: str,
        coords: list[int],
        function: Type2Function | Type3Function,
        extend_before: bool,
        extend_after: bool,
    ) -> None: ...

class Gradient(ABC):
    color_space: str
    colors: list[Incomplete]
    background: Incomplete | None
    extend_before: Incomplete
    extend_after: Incomplete
    bounds: Incomplete
    functions: Incomplete
    pattern: Pattern
    coords: Incomplete | None
    shading_type: int

    def __init__(self, colors, background, extend_before, extend_after, bounds): ...
    def get_shading_object(self) -> Shading: ...
    def get_pattern(self) -> Pattern: ...

class LinearGradient(Gradient):
    coords: list[str]
    shading_type: int
    def __init__(
        self,
        fpdf: FPDF,
        from_x: float,
        from_y: float,
        to_x: float,
        to_y: float,
        colors: list[Incomplete],
        background=None,
        extend_before: bool = False,
        extend_after: bool = False,
        bounds: list[int] | None = None,
    ) -> None: ...

class RadialGradient(Gradient):
    coords: list[str]
    shading_type: int
    def __init__(
        self,
        fpdf: FPDF,
        start_circle_x: float,
        start_circle_y: float,
        start_circle_radius: float,
        end_circle_x: float,
        end_circle_y: float,
        end_circle_radius: float,
        colors: list[Incomplete],
        background=None,
        extend_before: bool = False,
        extend_after: bool = False,
        bounds: list[int] | None = None,
    ): ...



"""Shared helpers for AST generation passes."""

from __future__ import annotations

from collections.abc import Sequence
from typing import Any, Generic, TypeVar

import jaclang.pycore.unitree as uni
from jaclang.pycore.passes.uni_pass import UniPass

T = TypeVar("T")
ChildPassT = TypeVar("ChildPassT", bound="BaseAstGenPass[Any]")


class BaseAstGenPass(UniPass, Generic[T]):
    """Common functionality shared across AST generation passes."""

    def _get_body_inner(
        self, node: uni.Archetype | uni.Enum | uni.Ability
    ) -> Sequence[uni.UniNode] | None:
        """Return the list of body statements regardless of ImplDef wrapping."""
        body = node.body
        if isinstance(body, uni.ImplDef) and isinstance(body.body, list):
            return body.body
        if isinstance(body, list):
            return body
        return None

    def _merge_module_bodies(self, node: uni.Module) -> list[uni.UniNode]:
        """Concatenate impl/test bodies with the main module body."""
        clean_body = [item for item in node.body if not isinstance(item, uni.ImplDef)]
        merged: list[uni.UniNode] = []
        for mod in node.impl_mod:
            merged.extend(mod.body)
        merged.extend(clean_body)
        for mod in node.test_mod:
            merged.extend(mod.body)
        return merged

    def _init_child_passes(self, pass_class: type[ChildPassT]) -> list[ChildPassT]:
        """Instantiate child passes for impl and test modules."""
        return [
            pass_class(ir_in=sub_module, prog=self.prog)
            for sub_module in self.ir_in.impl_mod + self.ir_in.test_mod
        ]

    def _flatten_ast_list(self, items: list[T | list[T] | None]) -> list[T]:
        """Flatten nested AST lists while skipping ``None`` entries."""
        flattened: list[T] = []
        for item in items:
            if isinstance(item, list):
                flattened.extend(item)
            elif item is not None:
                flattened.append(item)
        return flattened



from typing import Final

def generate_test_payment_method_code(number): ...

VisaPaymentMethodCode: Final = "stub-4111111111111111"
InvalidPaymentMethodCode: Final = "stub-invalid-payment-method-code"

Session: Final = "stub-session"
InvalidSession: Final = "stub-invalid-session"



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["min_weighted_vertex_cover"]

@_dispatchable
def min_weighted_vertex_cover(G: Graph[_Node], weight: str | None = None) -> set[Incomplete]: ...



from typing import Final

from ._config import options as options
from ._exports import (
    gpd as gpd,
    list_layers as list_layers,
    np as np,
    pd as pd,
    read_feather as read_feather,
    read_file as read_file,
    read_parquet as read_parquet,
    read_postgis as read_postgis,
)
from .array import points_from_xy as points_from_xy
from .geodataframe import GeoDataFrame as GeoDataFrame
from .geoseries import GeoSeries as GeoSeries
from .tools import clip as clip, overlay as overlay, sjoin as sjoin, sjoin_nearest as sjoin_nearest
from .tools._show_versions import show_versions as show_versions

__version__: Final[str]



from typing import Final

from .client import Client as Client
from .exceptions import (
    BadRequestsError as BadRequestsError,
    ForbiddenError as ForbiddenError,
    GatewayTimeoutError as GatewayTimeoutError,
    HTTPError as HTTPError,
    InternalServerError as InternalServerError,
    MethodNotAllowedError as MethodNotAllowedError,
    NotFoundError as NotFoundError,
    PayloadTooLargeError as PayloadTooLargeError,
    ServiceUnavailableError as ServiceUnavailableError,
    TooManyRequestsError as TooManyRequestsError,
    UnauthorizedError as UnauthorizedError,
    UnsupportedMediaTypeError as UnsupportedMediaTypeError,
)

dir_path: Final[str]
__version__: Final[str]



from hvac.api.vault_api_base import VaultApiBase

SUPPORTED_MFA_TYPES: list[str]
SUPPORTED_AUTH_METHODS: list[str]

class LegacyMfa(VaultApiBase):
    def configure(self, mount_point, mfa_type: str = "duo", force: bool = False): ...
    def read_configuration(self, mount_point): ...
    def configure_duo_access(self, mount_point, host, integration_key, secret_key): ...
    def configure_duo_behavior(self, mount_point, push_info=None, user_agent=None, username_format: str = "%s"): ...
    def read_duo_behavior_configuration(self, mount_point): ...



from _typeshed import Incomplete
from collections.abc import Collection, Hashable, Iterable, Mapping, Sequence
from typing import Literal, overload
from typing_extensions import TypeAlias

import numpy as np
import pandas as pd
from matplotlib.axes import Axes  # type: ignore[import-not-found]
from matplotlib.colors import Colormap, Normalize  # type: ignore[import-not-found]
from matplotlib.typing import ColorType  # type: ignore[import-not-found]
from numpy.typing import ArrayLike, NDArray
from pandas.plotting import PlotAccessor

from .geodataframe import GeoDataFrame
from .geoseries import GeoSeries

_ColorOrColors: TypeAlias = ColorType | Sequence[ColorType] | ArrayLike

def plot_series(
    s: GeoSeries,
    cmap: str | Colormap | None = None,
    color: _ColorOrColors | None = None,
    ax: Axes | None = None,
    figsize: tuple[float, float] | None = None,
    aspect: Literal["auto", "equal"] | float | None = "auto",
    autolim: bool = True,
    *,
    # Extracted from `**style_kwds`
    vmin: float = ...,
    vmax: float = ...,
    facecolor: _ColorOrColors | None = None,
    norm: Normalize | None = None,
    **style_kwds,
) -> Axes: ...

# IMPORTANT: keep roughly in sync with `GeoplotAccessor` methods below
def plot_dataframe(
    df: GeoDataFrame,
    column: Hashable | None = None,
    cmap: str | Colormap | None = None,
    color: _ColorOrColors | None = None,
    ax: Axes | None = None,
    cax: Axes | None = None,
    categorical: bool = False,
    legend: bool = False,
    scheme: str | None = None,
    k: int = 5,
    vmin: float | None = None,
    vmax: float | None = None,
    markersize: str | float | Iterable[float] | ArrayLike | None = None,
    figsize: tuple[float, float] | None = None,
    legend_kwds: dict[str, Incomplete] | None = None,
    categories: Iterable[Hashable] | None = None,
    classification_kwds: dict[str, Incomplete] | None = None,
    missing_kwds: dict[str, Incomplete] | None = None,
    aspect: Literal["auto", "equal"] | float | None = "auto",
    autolim: bool = True,
    *,
    # Extracted from `**style_kwds`
    norm: Normalize | None = None,
    alpha: float = 1,
    facecolor: _ColorOrColors | None = None,
    edgecolor: _ColorOrColors | None = None,
    linewidth: float = ...,
    label: str = "NaN",
    **style_kwds,
) -> Axes: ...

# IMPORTANT: keep roughly in sync with `plot_dataframe`
class GeoplotAccessor(PlotAccessor):
    # The first 3 overloads of calls are from pandas, the last overload is geopandas specific
    @overload  # type: ignore[override]
    def __call__(
        self,
        x: Hashable = ...,
        y: Hashable | Sequence[Hashable] = ...,
        *,
        kind: Literal["line", "bar", "barh", "hist", "box", "kde", "density", "area", "pie", "scatter", "hexbin"],
        ax: Axes | None = None,
        subplots: Literal[False] = False,
        sharex: bool | None = None,
        sharey: bool | None = None,
        layout: tuple[int, int] | None = None,
        figsize: tuple[float, float] | None = None,
        use_index: bool = True,
        title: str | None = None,
        grid: bool | None = None,
        legend: bool | Literal["reverse"] = True,
        style: str | Sequence[str] | Mapping[Incomplete, str] | None = None,
        logx: bool | Literal["sym"] = False,
        logy: bool | Literal["sym"] = False,
        loglog: bool | Literal["sym"] = False,
        xticks: Sequence[float] | None = None,
        yticks: Sequence[float] | None = None,
        xlim: tuple[float, float] | list[float] | None = None,
        ylim: tuple[float, float] | list[float] | None = None,
        xlabel: str | None = None,
        ylabel: str | None = None,
        rot: float | None = None,
        fontsize: float | None = None,
        cmap: str | Colormap | None = None,  # also accepts `colormap` but plot_dataframe uses `cmap`
        colorbar: bool | None = None,
        position: float = 0.5,
        table: bool | pd.Series[Incomplete] | pd.DataFrame = False,
        yerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        xerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        stacked: bool = ...,  # default value depends on kind
        secondary_y: bool | Sequence[Hashable] = False,
        mark_right: bool = True,
        include_bool: bool = False,
        backend: str | None = None,
        **kwargs,
    ) -> Axes: ...
    @overload
    def __call__(
        self,
        x: Hashable = ...,
        y: Hashable | Sequence[Hashable] = ...,
        *,
        kind: Literal["line", "bar", "barh", "hist", "kde", "density", "area", "pie", "scatter", "hexbin"],
        ax: Sequence[Axes] | None = None,
        subplots: Literal[True] | Iterable[Iterable[Hashable]],
        sharex: bool | None = None,
        sharey: bool | None = None,
        layout: tuple[int, int] | None = None,
        figsize: tuple[float, float] | None = None,
        use_index: bool = True,
        title: str | Collection[str] | None = None,
        grid: bool | None = None,
        legend: bool | Literal["reverse"] = True,
        style: str | Sequence[str] | Mapping[Incomplete, str] | None = None,
        logx: bool | Literal["sym"] = False,
        logy: bool | Literal["sym"] = False,
        loglog: bool | Literal["sym"] = False,
        xticks: Sequence[float] | None = None,
        yticks: Sequence[float] | None = None,
        xlim: tuple[float, float] | list[float] | None = None,
        ylim: tuple[float, float] | list[float] | None = None,
        xlabel: str | None = None,
        ylabel: str | None = None,
        rot: float | None = None,
        fontsize: float | None = None,
        cmap: str | Colormap | None = None,  # also accepts `colormap` but plot_dataframe uses `cmap`
        colorbar: bool | None = None,
        position: float = 0.5,
        table: bool | pd.Series[Incomplete] | pd.DataFrame = False,
        yerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        xerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        stacked: bool = ...,  # default value depends on kind
        secondary_y: bool | Sequence[Hashable] = False,
        mark_right: bool = True,
        include_bool: bool = False,
        backend: str | None = None,
        **kwargs,
    ) -> NDArray[np.object_]: ...  # should be NDArray[Axes] but it is not supported
    @overload
    def __call__(
        self,
        x: Hashable = ...,
        y: Hashable | Sequence[Hashable] = ...,
        *,
        kind: Literal["box"],
        ax: Sequence[Axes] | None = None,
        subplots: Literal[True] | Iterable[Iterable[Hashable]],
        sharex: bool | None = None,
        sharey: bool | None = None,
        layout: tuple[int, int] | None = None,
        figsize: tuple[float, float] | None = None,
        use_index: bool = True,
        title: str | Collection[str] | None = None,
        grid: bool | None = None,
        legend: bool | Literal["reverse"] = True,
        style: str | Sequence[str] | Mapping[Incomplete, str] | None = None,
        logx: bool | Literal["sym"] = False,
        logy: bool | Literal["sym"] = False,
        loglog: bool | Literal["sym"] = False,
        xticks: Sequence[float] | None = None,
        yticks: Sequence[float] | None = None,
        xlim: tuple[float, float] | list[float] | None = None,
        ylim: tuple[float, float] | list[float] | None = None,
        xlabel: str | None = None,
        ylabel: str | None = None,
        rot: float | None = None,
        fontsize: float | None = None,
        cmap: str | Colormap | None = None,  # also accepts `colormap` but plot_dataframe uses `cmap`
        colorbar: bool | None = None,
        position: float = 0.5,
        table: bool | pd.Series[Incomplete] | pd.DataFrame = False,
        yerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        xerr: pd.DataFrame | pd.Series[float] | ArrayLike | Mapping[Incomplete, ArrayLike] | str = ...,
        stacked: bool = ...,  # default value depends on kind
        secondary_y: bool | Sequence[Hashable] = False,
        mark_right: bool = True,
        include_bool: bool = False,
        backend: str | None = None,
        **kwargs,
    ) -> pd.Series[Axes]: ...  # type: ignore[type-var] # pyright: ignore[reportInvalidTypeArguments]
    @overload
    def __call__(
        self,
        column: Hashable | pd.Series | pd.Index | NDArray | None = None,
        cmap: str | Colormap | None = None,
        color: _ColorOrColors | None = None,
        ax: Axes | None = None,
        cax: Axes | None = None,
        categorical: bool = False,
        legend: bool = False,
        scheme: str | None = None,
        k: int = 5,
        vmin: float | None = None,
        vmax: float | None = None,
        markersize: str | float | Iterable[float] | ArrayLike | None = None,
        figsize: tuple[float, float] | None = None,
        legend_kwds: dict[str, Incomplete] | None = None,
        categories: Iterable[Hashable] | None = None,
        classification_kwds: dict[str, Incomplete] | None = None,
        missing_kwds: dict[str, Incomplete] | None = None,
        aspect: Literal["auto", "equal"] | float | None = "auto",
        *,
        kind: Literal["geo"] = "geo",
        # Extracted from `**style_kwds`
        norm: Normalize | None = None,
        alpha: float = 1,
        facecolor: _ColorOrColors | None = None,
        edgecolor: _ColorOrColors | None = None,
        linewidth: float = ...,
        label: str = "NaN",
        **style_kwds,
    ) -> Axes: ...
    def geo(
        self,
        column: Hashable | pd.Series | pd.Index | NDArray | None = None,
        cmap: str | Colormap | None = None,
        color: _ColorOrColors | None = None,
        ax: Axes | None = None,
        cax: Axes | None = None,
        categorical: bool = False,
        legend: bool = False,
        scheme: str | None = None,
        k: int = 5,
        vmin: float | None = None,
        vmax: float | None = None,
        markersize: str | float | Iterable[float] | ArrayLike | None = None,
        figsize: tuple[float, float] | None = None,
        legend_kwds: dict[str, Incomplete] | None = None,
        categories: Iterable[Hashable] | None = None,
        classification_kwds: dict[str, Incomplete] | None = None,
        missing_kwds: dict[str, Incomplete] | None = None,
        aspect: Literal["auto", "equal"] | float | None = "auto",
        *,
        # Extracted from `**style_kwds`
        norm: Normalize | None = None,
        alpha: float = 1,
        facecolor: _ColorOrColors | None = None,
        edgecolor: _ColorOrColors | None = None,
        linewidth: float = ...,
        label: str = "NaN",
        **style_kwds,
    ) -> Axes: ...



from collections.abc import Iterable
from typing import Any, Literal, TypedDict, overload, type_check_only
from typing_extensions import NotRequired, TypeAlias

from asgiref.typing import ASGIVersions
from channels.testing.application import ApplicationCommunicator
from channels.utils import _ChannelApplication

@type_check_only
class _WebsocketTestScope(TypedDict, total=False):
    spec_version: int
    type: Literal["websocket"]
    asgi: ASGIVersions
    http_version: str
    scheme: str
    path: str
    raw_path: bytes
    query_string: bytes
    root_path: str
    headers: Iterable[tuple[bytes, bytes]] | None
    client: tuple[str, int] | None
    server: tuple[str, int | None] | None
    subprotocols: Iterable[str] | None
    state: NotRequired[dict[str, Any]]
    extensions: dict[str, dict[object, object]] | None

_Connected: TypeAlias = bool
_CloseCodeOrAcceptSubProtocol: TypeAlias = int | str | None
_WebsocketConnectResponse: TypeAlias = tuple[_Connected, _CloseCodeOrAcceptSubProtocol]

class WebsocketCommunicator(ApplicationCommunicator):
    scope: _WebsocketTestScope
    response_headers: list[tuple[bytes, bytes]] | None

    def __init__(
        self,
        application: _ChannelApplication,
        path: str,
        headers: Iterable[tuple[bytes, bytes]] | None = None,
        subprotocols: Iterable[str] | None = None,
        spec_version: int | None = None,
    ) -> None: ...
    async def connect(self, timeout: float = 1) -> _WebsocketConnectResponse: ...
    async def send_to(self, text_data: str | None = None, bytes_data: bytes | None = None) -> None: ...
    async def receive_from(self, timeout: float = 1) -> str | bytes: ...

    # These overloads reflect common usage, where users typically send and receive `dict[str, Any]`.
    # The base case allows `Any` to support broader `json.dumps` / `json.loads` compatibility.
    @overload
    async def send_json_to(self, data: dict[str, Any]) -> None: ...
    @overload
    async def send_json_to(self, data: Any) -> None: ...
    async def receive_json_from(self, timeout: float = 1) -> Any: ...
    async def disconnect(self, code: int = 1000, timeout: float = 1) -> None: ...



from logging import Logger

LOG: Logger
__version__: str
__version_info__: tuple[int, int, int]
LOG_FORMAT: str

def configure_logging(verbosity: int, filename: str | None = None, logformat: str = ...) -> None: ...



from abc import ABCMeta, abstractmethod
from collections.abc import Callable, Iterable, Sequence
from typing import Any, Literal
from typing_extensions import Self, TypeAlias

import tensorflow as tf
from tensorflow import Operation, Tensor
from tensorflow._aliases import DTypeLike, KerasSerializable, TensorCompatible
from tensorflow.keras.initializers import _Initializer

_Output: TypeAlias = Tensor | dict[str, Tensor]

class Metric(tf.keras.layers.Layer[tf.Tensor, tf.Tensor], metaclass=ABCMeta):
    def __init__(self, dtype: DTypeLike | None = None, name: str | None = None) -> None: ...
    def __new__(cls, *args: Any, **kwargs: Any) -> Self: ...
    def reset_state(self) -> None: ...
    @abstractmethod
    def update_state(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Operation | None: ...
    @abstractmethod
    def result(self) -> _Output: ...
    # Metric inherits from keras.Layer, but its add_weight method is incompatible with the one from "Layer".
    def add_weight(  # type: ignore[override]
        self,
        name: str,
        shape: Iterable[int | None] | None = (),
        aggregation: tf.VariableAggregation = ...,
        synchronization: tf.VariableSynchronization = ...,
        initializer: _Initializer | None = None,
        dtype: DTypeLike | None = None,
    ) -> None: ...

class AUC(Metric):
    _from_logits: bool
    _num_labels: int
    num_labels: int | None
    def __init__(
        self,
        num_thresholds: int = 200,
        curve: Literal["ROC", "PR"] = "ROC",
        summation_method: Literal["interpolation", "minoring", "majoring"] = "interpolation",
        name: str | None = None,
        dtype: DTypeLike | None = None,
        thresholds: Sequence[float] | None = None,
        multi_label: bool = False,
        num_labels: int | None = None,
        label_weights: TensorCompatible | None = None,
        from_logits: bool = False,
    ) -> None: ...
    def update_state(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Operation: ...
    def result(self) -> tf.Tensor: ...

class Precision(Metric):
    def __init__(
        self,
        thresholds: float | Sequence[float] | None = None,
        top_k: int | None = None,
        class_id: int | None = None,
        name: str | None = None,
        dtype: DTypeLike | None = None,
    ) -> None: ...
    def update_state(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Operation: ...
    def result(self) -> tf.Tensor: ...

class Recall(Metric):
    def __init__(
        self,
        thresholds: float | Sequence[float] | None = None,
        top_k: int | None = None,
        class_id: int | None = None,
        name: str | None = None,
        dtype: DTypeLike | None = None,
    ) -> None: ...
    def update_state(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Operation: ...
    def result(self) -> tf.Tensor: ...

class MeanMetricWrapper(Metric):
    def __init__(
        self, fn: Callable[[tf.Tensor, tf.Tensor], tf.Tensor], name: str | None = None, dtype: DTypeLike | None = None
    ) -> None: ...
    def update_state(
        self, y_true: TensorCompatible, y_pred: TensorCompatible, sample_weight: TensorCompatible | None = None
    ) -> Operation: ...
    def result(self) -> tf.Tensor: ...

class BinaryAccuracy(MeanMetricWrapper):
    def __init__(self, name: str | None = "binary_accuracy", dtype: DTypeLike | None = None, threshold: float = 0.5) -> None: ...

class Accuracy(MeanMetricWrapper):
    def __init__(self, name: str | None = "accuracy", dtype: DTypeLike | None = None) -> None: ...

class CategoricalAccuracy(MeanMetricWrapper):
    def __init__(self, name: str | None = "categorical_accuracy", dtype: DTypeLike | None = None) -> None: ...

class TopKCategoricalAccuracy(MeanMetricWrapper):
    def __init__(self, k: int = 5, name: str | None = "top_k_categorical_accuracy", dtype: DTypeLike | None = None) -> None: ...

class SparseTopKCategoricalAccuracy(MeanMetricWrapper):
    def __init__(
        self, k: int = 5, name: str | None = "sparse_top_k_categorical_accuracy", dtype: DTypeLike | None = None
    ) -> None: ...

def serialize(metric: KerasSerializable) -> dict[str, Any]: ...
def binary_crossentropy(
    y_true: TensorCompatible, y_pred: TensorCompatible, from_logits: bool = False, label_smoothing: float = 0.0, axis: int = -1
) -> Tensor: ...
def categorical_crossentropy(
    y_true: TensorCompatible, y_pred: TensorCompatible, from_logits: bool = False, label_smoothing: float = 0.0, axis: int = -1
) -> Tensor: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed.wsgi import WSGIApplication
from typing import Any

from fanstatic.core import Resource
from fanstatic.injector import InjectorPlugin
from fanstatic.publisher import Delegator
from webob import Request, Response
from webob.dec import wsgify

def Fanstatic(
    app: WSGIApplication, publisher_signature: str = "fanstatic", injector: InjectorPlugin | None = None, **config: Any
) -> Delegator: ...
def make_fanstatic(app: WSGIApplication, global_config: Any, **local_config: Any) -> Delegator: ...

class Serf:
    resource: Resource
    def __init__(self, resource: Resource) -> None: ...
    @wsgify
    def __call__(self, request: Request) -> Response: ...

def make_serf(global_config: Any, **local_config: Any) -> Serf: ...
def resolve(name: str, module: str | None = None) -> Any: ...



from typing import Literal

from ..extractor.common import _InfoDict
from .fragment import FragmentFD

class DashSegmentsFD(FragmentFD):
    FD_NAME: Literal["dashsegments"]
    def real_download(self, filename: str, info_dict: _InfoDict) -> bool: ...



from typing import Final

GRPC_GENERATED_VERSION: Final[str]
GRPC_VERSION: Final[str]

class HealthStub:
    def __init__(self, channel) -> None: ...

class HealthServicer:
    def Check(self, request, context): ...
    def Watch(self, request, context): ...

def add_HealthServicer_to_server(servicer, server) -> None: ...

class Health:
    @staticmethod
    def Check(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ): ...
    @staticmethod
    def Watch(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ): ...



import datetime
from typing import Any
from typing_extensions import TypeAlias

from .core import UnitedStates

_HebrewDate: TypeAlias = Any  # from `pyluach.dates` package

class HebrewHolidays:
    hebrew_calendars: dict[int, list[tuple[_HebrewDate | None, datetime.date]]]
    @classmethod
    def get_hebrew_calendar(cls, gregorian_year: int) -> list[tuple[_HebrewDate | None, datetime.date]]: ...
    @classmethod
    def search_hebrew_calendar(cls, gregorian_year: int, hebrew_month: int, hebrew_day: int) -> datetime.date: ...
    @classmethod
    def get_rosh_hashanah(cls, year: int) -> datetime.date: ...
    @classmethod
    def get_yom_kippur(cls, year: int) -> datetime.date: ...

class Florida(UnitedStates): ...

class FloridaLegal(Florida):
    def __init__(self, *args, **kwargs) -> None: ...

class FloridaCircuitCourts(HebrewHolidays, Florida): ...
class FloridaMiamiDade(Florida): ...



import asyncio
from concurrent.futures import Future
from typing import Dict

from lsprotocol.types import (
    PROGRESS,
    WINDOW_WORK_DONE_PROGRESS_CREATE,
    ProgressParams,
    ProgressToken,
    WorkDoneProgressBegin,
    WorkDoneProgressEnd,
    WorkDoneProgressReport,
    WorkDoneProgressCreateParams,
)
from pygls.protocol import LanguageServerProtocol


class Progress:
    """A class for working with client's progress bar.

    Attributes:
        _lsp(LanguageServerProtocol): Language server protocol instance
        tokens(dict): Holds futures for work done progress tokens that are
            already registered. These futures will be cancelled if the client
            sends a cancel work done process notification.
    """

    def __init__(self, lsp: LanguageServerProtocol) -> None:
        self._lsp = lsp

        self.tokens: Dict[ProgressToken, Future] = {}

    def _check_token_registered(self, token: ProgressToken) -> None:
        if token in self.tokens:
            raise Exception("Token is already registered!")

    def _register_token(self, token: ProgressToken) -> None:
        self.tokens[token] = Future()

    def create(self, token: ProgressToken, callback=None) -> Future:
        """Create a server initiated work done progress."""
        self._check_token_registered(token)

        def on_created(*args, **kwargs):
            self._register_token(token)
            if callback is not None:
                callback(*args, **kwargs)

        return self._lsp.send_request(
            WINDOW_WORK_DONE_PROGRESS_CREATE,
            WorkDoneProgressCreateParams(token=token),
            on_created,
        )

    async def create_async(self, token: ProgressToken) -> asyncio.Future:
        """Create a server initiated work done progress."""
        self._check_token_registered(token)

        result = await self._lsp.send_request_async(
            WINDOW_WORK_DONE_PROGRESS_CREATE,
            WorkDoneProgressCreateParams(token=token),
        )
        self._register_token(token)
        return result

    def begin(self, token: ProgressToken, value: WorkDoneProgressBegin) -> None:
        """Notify beginning of work."""
        # Register cancellation future for the case of client initiated progress
        self.tokens.setdefault(token, Future())

        return self._lsp.notify(PROGRESS, ProgressParams(token=token, value=value))

    def report(self, token: ProgressToken, value: WorkDoneProgressReport) -> None:
        """Notify progress of work."""
        self._lsp.notify(PROGRESS, ProgressParams(token=token, value=value))

    def end(self, token: ProgressToken, value: WorkDoneProgressEnd) -> None:
        """Notify end of work."""
        self._lsp.notify(PROGRESS, ProgressParams(token=token, value=value))
